
/* 
 * GeomView.m -- Implementation file for the GeomView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, or even that it has a use.
 *
 */

#import "geom.h"
#import "sulf.h"
#import "pen.h"
#import "tri.h"
#import "GeomView.h"
#import "drawingFuncs.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h> 

#define MAXNUMLENGTH 50
#define DEFAULTRADIUS 1.0
#define COEPHS 25
#define POYNTS 12

int    drawView=0;
int    multiView=0;
int    degree=0;
int    drawBack=YES;
int    drawContour=NO;
int    drawEditedMolecule=NO;
int    drawFatMolecule=NO;
int    drawInspectedMolecule=NO;
int    drawPenMatrix=NO;
int    drawSulfMatrix=NO;
int    drawTriMatrix=NO;
int    drawPenMode=NO;
int    drawSulfMode=NO;
int    drawTriMode=NO;
int    drawSphere=NO;
int    drawCutting=NO;
int    drawReflect=NO;
int    drawSplit=NO;
int    drawShadows=NO;
int    drawAxSphere=NO;
int    drawEuSphere=NO;
int    drawUnisphere=NO;
int    drawDisphere=NO;
int    runCycle=1;
int    Vmode, Vpall;
int    nlat, nlong, ntot, clat, clong;

double coearry[COEPHS][2];
double master[POYNTS+1][2];
double poiarry[POYNTS][2];
double valarry[POYNTS][2];
double spRadius;

double xx, yy, zz;

extern int    bbe[12][12];
extern int    sspch, sslen;
extern int    sglon, sglat, sglen;
extern int    slthe, slphi, sllen;
extern int    frdepth;
extern int    tu[10], abe[12], afc[21];


extern double r0[3], r, a0[3],      m0[4];
extern double o[3][3],  mrot[3][3], s[5][8], t[5][8], u[10][8], uu[3][3];
extern double v[][8],   su[][8],    fc[][8], be[][8], rp[][8],  rl[][8];
extern double qs[5][8], qt[5][8],   qang[5];
extern double rf[][8],  mp[][8];

extern double bep[];
extern double sup[];
extern double fcp[];
extern double mpp[];

extern double suls[][3];		/* skeletal array	*/
extern double sudb[][7][3][3];		/* diagonal band	*/
extern double sudy[][11][3][3];		/* dynamical martrix	*/
extern double sunm[][11][3][3];		/* eigenvector martrix	*/
extern double supl[8];			/* parameter list	*/
       double sudi[11][11][3][3];	/* save dyn matrix	*/

extern double trdb[TDM][TDB];		/* diagonal band	*/
extern double trdm[TDM][TDM];		/* dynamical matrix	*/
extern double trnm[TDM][TDM];		/* normal modes		*/
extern double trpl[8];			/* parameter list	*/
       double trdi[TDM][TDM];		/* save dyn. matrix	*/

extern double pedb[PDM][PDB];		/* diagonal band	*/
extern double pedy[PDM][PDM];		/* dynamical matrix	*/
extern double penm[PDM][PDM];		/* normal modes		*/
extern double pepl[8];			/* parameter list	*/
       double pedi[PDM][PDM];		/* save dyn. matrix	*/

double qpole[3][3], qdiag[3][3];

double cubic[25][25][25];

extern float getNumber(NXStream *stream);
extern int getSeparator(NXStream *stream);


@implementation GeomView


/* - - - - - - - - - - -  V I D E O  - - - - - - - - - - - - - - - - - */


void videocursor(p,i,j) int p, i, j; {}
void videoputc(c,i) char c; int i; {}
void videomode(i) int i; {}
void videopalette(i) int i; {}
void videoscroll(i,j,k,l,d,c) int i, j, k, l, d, c; {}
void videocattr(p,c,n,l) int p, c, n, l; {}
void videoclear(){}
void videobackground(int c){}

void videodot(i,j,l) int i, j, l; {
  switch (l) {
    case 0: if (drawView==2) PSsetrgbcolor(0.7,1.0,0.6); else PSsetrgbcolor(1.0,1.0,1.0); break;
    case 1: PSsetrgbcolor(0.5,0.0,0.0); break;
    case 3: PSsetrgbcolor(0.0,0.5,0.0); break;
    case 2: PSsetrgbcolor(0.0,0.0,0.5); break;
    default: PSsetrgbcolor(0.1,0.1,0.1); break;}
/*  PSsetgray(NX_BLACK); */
 
/*   drawCircle(y,x,1.0); */

    switch (drawView) {
      case 0: PSrectfill((float)j,(float)i,1.0,1.0); break;
      case 1: PSrectfill(2.0*(float)i,398.0-2.0*(float)j,1.25,1.25); break;
      case 2: PSrectfill(8.0*(float)j+1.5,60.0-8.0*(float)i,4.75,4.75); break;
      case 3: PSrectfill(4.0*(float)j+2.0,48.0-4.0*(float)i,3.0,3.0); break;
      case 4: PSrectfill((float)j,199.0-(float)i,1.25,1.25); break;
      case 5: if (l==0) PSmoveto((float)j,(float)i);
                 else PSlineto((float)j,(float)i);
     default: break;}
      

}

void videofdot(x,y,l) double x, y; int l; {
    switch (drawView) {
      case 0: if (l!=0) PSrectfill(y,x,1.0,1.0); break;
      case 5: if (l==0) PSmoveto(y,x); else PSlineto(y,x); break;
     default: break;}
}

void stereofdot(x,y,z,l) double x, y, z; int l; {
    switch (drawView) {
      case 0: if (l!=0) 
		PSrectfill(x-150.0,z,1.0,1.0); 
		PSrectfill(y+150.0,z,1.0,1.0); 
		break;
      case 5: if (l==0) {xx=x; yy=y; zz=z;} else {
		PSmoveto(xx-150.0,zz); PSlineto(x-150.0,z);
		PSmoveto(yy+150.0,zz); PSlineto(y+150.0,z);
		xx=x; yy=y; zz=z;
		} break;
     default: break;}
}

void fsifdot(x,y,z,l,m) double x, y, z; int l, m; {
    x*=50.0; y*=50.0; z*=50.0;
    switch (drawView) {
      case 0:  
		if (l!=0) PSrectfill(x-150.0,z,1.0,1.0); 
		if (m!=0) PSrectfill(y+150.0,z,1.0,1.0); 
		break;
      case 5: 
		if (l!=0) {PSmoveto(xx-150.0,zz); PSlineto(x-150.0,z);}
		if (m!=0) {PSmoveto(yy+150.0,zz); PSlineto(y+150.0,z);}
		xx=x; yy=y; zz=z;
		break;
     default: break;}
}

/* - - - - - - - - - - - - -  G E O M  - - - - - - - - - - - - - - - - - - */

- appWillInit:sender {printf("appWillInit = geom\n"); return self;}

- appDidInit:sender {geoii(); return self;}

- printGEOM:sender {[keyWindow smartPrintPSCode: sender]; return self;}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyGEOM:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printGEOM:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyGEOM:sender {
    drawBack=NO;
    if (keyWindow==benzWindow)
	[self copySomeView: 5: &drawFatMolecule: theBenzView];
    else if (keyWindow==contWindow)
	[self copySomeView: 5: &drawContour: theContView];
    else if (keyWindow==diSpWindow)
	[self copySomeView: 5: &drawDisphere: theDiSpView];
    else if (keyWindow==feroWindow)
	[self copySomeView: 5: &drawFatMolecule: theFeroView];
    else if (keyWindow==geomWindow)
	[self copySomeView: 5: &drawSphere: theGeomView];
    else if (keyWindow==infoPanel)
	[self copySomeView: 0: &drawSphere: [infoPanel contentView]];
    else if (keyWindow==pentaWindow)
	[self copySomeView: 5: &drawPenMode: thePentaView];
    else if (keyWindow==polymWindow)
	[self copySomeView: 5: &drawFatMolecule: thePolymView];
    else if (keyWindow==quarWindow)
	[self copySomeView: 5: &drawCutting: theQuarView];
    else if (keyWindow==reflWindow)
	[self copySomeView: 5: &drawReflect: theReflView];
    else if (keyWindow==shadWindow)
	[self copySomeView: 5: &drawShadows: theShadView];
    else if (keyWindow==sulfWindow)
	[self copySomeView: 5: &drawFatMolecule: theSulfView];
    else if (keyWindow==thioWindow)
	{runCycle=1; [self copySomeView: 5: &drawSulfMode: theThioView];}
    else if (keyWindow==tridlWindow)
	[self copySomeView: 5: &drawTriMode: theTridlView];
    else if (keyWindow==unisWindow)
	[self copySomeView: 0: &drawUnisphere: theUnisView];
    drawBack=YES;
return self;
}

- copySomeView:(int)viewnum:(int *)draw:theView {
NXRect zbounds;
id     pBoard;
  drawView=viewnum;
  *draw=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theView getBounds: &zbounds];
  [theView writePSCodeInside: &zbounds to: pBoard]; 
  *draw=NO;
return self;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

- readEulerAngles:sender {
  a0[0]=360.0*[[euRot cellAt:0:0] doubleValue];
  a0[1]=360.0*[[euRot cellAt:1:0] doubleValue];
  a0[2]=360.0*[[euRot cellAt:2:0] doubleValue];
return self;
}

- readAngleAxis:(double *)a:(double *)aa {
  a[0]=[[cosRot cellAt:0:0] doubleValue];
  a[1]=[[cosRot cellAt:1:0] doubleValue];
  a[2]=[[cosRot cellAt:2:0] doubleValue];
  *aa=180.0*[angRot doubleValue];
  if (a[0]==0.0&&a[1]==0.0&a[2]==0.0) a[0]=a[1]=a[2]=0.886;
return self;
}

- readParameters:sender {int i, j; double o[3][3], a[3];
  i=0;
  if (strcmp([[sender selectedCell] title],"sphere 1")==0) i=0;
  if (strcmp([[sender selectedCell] title],"sphere 2")==0) i=1;
  if (strcmp([[sender selectedCell] title],"sphere 3")==0) i=2;
  if (strcmp([[sender selectedCell] title],"sphere 4")==0) i=3;
  if (strcmp([[sender selectedCell] title],"sphere 5")==0) i=4;
  if (strcmp([[sender selectedCell] title],"sphere 6")==0) i=5;
  for (j=0; j<3; j++) [[rightCenter cellAt:j:0] setDoubleValue:u[i][j]];
  [rightRadius setDoubleValue:u[i][3]];
  for (j=0; j<3; j++) [[rightEuler cellAt:j:0] setDoubleValue:u[i][j+4]];
spheu(o,&u[i][4]);
sphdc(a,o);
  [[rightAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[rightAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[rightAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[rightAngles cellAt:3:0] setDoubleValue:sphar(o)]; 
return self;
}

- readQuartet:sender {int i, j; double o[3][3], a[3];
  i=0;
  if (strcmp([[sender selectedCell] title],"sphere 1")==0) i=0;
  if (strcmp([[sender selectedCell] title],"sphere 2")==0) i=1;
  if (strcmp([[sender selectedCell] title],"sphere 3")==0) i=2;
  if (strcmp([[sender selectedCell] title],"sphere 4")==0) i=3;
  if (strcmp([[sender selectedCell] title],"sphere 5")==0) i=4;
  for (j=0; j<3; j++) [[rightCenter cellAt:j:0] setDoubleValue:qt[i][j]];
  [rightRadius setDoubleValue:qt[i][3]];
  for (j=0; j<3; j++) [[rightEuler cellAt:j:0] setDoubleValue:qt[i][j+4]];
  [cutAngle setDoubleValue:qang[i]];
  [cutSlider setDoubleValue:qang[i]];  
spheu(o,&qt[i][4]);
sphdc(a,o);
  [[rightAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[rightAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[rightAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[rightAngles cellAt:3:0] setDoubleValue:sphar(o)]; 
return self;
}

- rotateMolecule:sender {
  m0[0]=[[cosRot cellAt:0:0] doubleValue];
  m0[1]=[[cosRot cellAt:1:0] doubleValue];
  m0[2]=[[cosRot cellAt:2:0] doubleValue];
  m0[3]=180.0*[angRot  doubleValue];
  if (m0[0]==0.0&&m0[1]==0.0&m0[2]==0.0) m0[0]=m0[1]=m0[2]=0.886;
  sphax(mrot,m0[3],m0[0],m0[1],m0[2]);

return self;
}

- setParameters:sender {int i, j; double o[3][3], a[3];
  i=0;
  if (strcmp([sphereMenu title],"sphere 1")==0) i=0;
  if (strcmp([sphereMenu title],"sphere 2")==0) i=1;
  if (strcmp([sphereMenu title],"sphere 3")==0) i=2;
  if (strcmp([sphereMenu title],"sphere 4")==0) i=3;
  if (strcmp([sphereMenu title],"sphere 5")==0) i=4;
  if (strcmp([sphereMenu title],"sphere 6")==0) i=5;
  for (j=0; j<3; j++) u[i][j]=[[rightCenter cellAt:j:0] doubleValue];
  u[i][3]=[rightRadius doubleValue];
  for (j=0; j<3; j++) u[i][j+4]=[[rightEuler cellAt:j:0] doubleValue];
spheu(o,&u[i][4]);
sphdc(a,o);
  [[rightAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[rightAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[rightAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[rightAngles cellAt:3:0] setDoubleValue:sphar(o)]; 
return self;
}

- setQuartet:sender {int i, j; double o[3][3], a[3];
  i=0;
  if (strcmp([sphereMenu title],"sphere 1")==0) i=0;
  if (strcmp([sphereMenu title],"sphere 2")==0) i=1;
  if (strcmp([sphereMenu title],"sphere 3")==0) i=2;
  if (strcmp([sphereMenu title],"sphere 4")==0) i=3;
  if (strcmp([sphereMenu title],"sphere 5")==0) i=4;
  for (j=0; j<3; j++) qt[i][j]=[[rightCenter cellAt:j:0] doubleValue];
  qt[i][3]=[rightRadius doubleValue];
  for (j=0; j<3; j++) qt[i][j+4]=[[rightEuler cellAt:j:0] doubleValue];
  qang[i]=[cutAngle doubleValue];
spheu(o,&qt[i][4]);
geoqr(qs,qt,5);
sphdc(a,o);
  [[rightAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[rightAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[rightAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[rightAngles cellAt:3:0] setDoubleValue:sphar(o)]; 
return self;
}

- gramCent:sender {int i, j; double xx, yy, zz, r;
for (i=0; i<5; i++) for (j=0; j<5; j++) {
  xx=qt[i][0]-qt[j][0]; xx*=xx;
  yy=qt[i][1]-qt[j][1]; yy*=yy;
  zz=qt[i][2]-qt[j][2]; zz*=zz;
  r=sqrt(xx+yy+zz);
  [[gramMatrix cellAt :i :j] setDoubleValue:r];
  }
return self;
}

- gramCosine:sender {int i, j; double xx, yy, zz, p, c, a, ri, rj;
for (i=0; i<5; i++) for (j=0; j<5; j++) {
  xx=qt[i][0]-qt[j][0]; xx*=xx;
  yy=qt[i][1]-qt[j][1]; yy*=yy;
  zz=qt[i][2]-qt[j][2]; zz*=zz;
  ri=qt[i][3]; rj=qt[j][3];
  p=0.5*(ri*ri+rj*rj-xx-yy-zz);
  c=(p/(ri*rj));
  if (c<-1.0||c>1.0) a=c; else a=(180.0*acos(c))/3.14159;
  [[gramMatrix cellAt :i :j] setDoubleValue:a];
  }
return self;
}

- gramProj:sender {int i, j; double xx, yy, zz, ri, rj;
for (i=0; i<5; i++) for (j=0; j<5; j++) {
  xx=qt[i][0]-qt[j][0]; xx*=xx;
  yy=qt[i][1]-qt[j][1]; yy*=yy;
  zz=qt[i][2]-qt[j][2]; zz*=zz;
  ri=qt[i][3]; rj=qt[j][3];
  [[gramMatrix cellAt :i :j] setDoubleValue:0.5*(ri*ri+rj*rj-xx-yy-zz)];
  }
return self;
}

- setLighting:sender {double a[3], aa;
  [self readAngleAxis:a:&aa];
  sphax(uu,aa,a[0],a[1],a[2]);
  [[leftAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[leftAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[leftAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[leftAngles cellAt:3:0] setDoubleValue:sphar(uu)]; 
return self;
}

- setRotation:sender {double t, a[3];
  switch ([sender selectedRow]) {
    case 0: [angRot setDoubleValue:90.0/180.0];
            [[cosRot cellAt:0:0] setDoubleValue:0.0];
            [[cosRot cellAt:1:0] setDoubleValue:1.0];
            [[cosRot cellAt:2:0] setDoubleValue:0.0];
            break;
    case 1: [angRot setDoubleValue:90.0/180.0];
            [[cosRot cellAt:0:0] setDoubleValue:0.0];
            [[cosRot cellAt:1:0] setDoubleValue:0.0];
            [[cosRot cellAt:2:0] setDoubleValue:1.0];
            break;
    case 2: [angRot setDoubleValue:90.0/180.0];
            [[cosRot cellAt:0:0] setDoubleValue:1.0];
            [[cosRot cellAt:1:0] setDoubleValue:0.0];
            [[cosRot cellAt:2:0] setDoubleValue:0.0];
            break;
    case 3: [angRot setDoubleValue:60.0/180.0];
            [[cosRot cellAt:0:0] setDoubleValue:0.3];
            [[cosRot cellAt:1:0] setDoubleValue:0.3];
            [[cosRot cellAt:2:0] setDoubleValue:0.3];
            break;
    case 4: t=sphar(qdiag)/180.0;
	    [angRot setDoubleValue:t];
	    sphdc(a,qdiag);
            [[cosRot cellAt:0:0] setDoubleValue:-a[0]];
            [[cosRot cellAt:1:0] setDoubleValue:-a[1]];
            [[cosRot cellAt:2:0] setDoubleValue:-a[2]];
            break;
    default:break;}
return self;
}

- showEditedMolecule:sender {
  drawView=5;
  drawEditedMolecule=YES;
  [self display];
  drawEditedMolecule=NO;
	needsClearing = YES;
return self;
}

- showFatMolecule:sender {
  drawView=5;
  drawFatMolecule=YES;
  [self display];
  drawFatMolecule=NO;
	needsClearing = YES;
return self;
}

- showInspectedMolecule:sender {
  drawView=5;
  drawInspectedMolecule=YES;
  [self display];
  drawInspectedMolecule=NO;
	needsClearing = YES;
return self;
}

- printEdited:sender {
  drawView=5;
  drawEditedMolecule=YES;
  [self printPSCode:self];
  drawEditedMolecule=NO;
	needsClearing = YES;
return self;
}

- showSphere:sender {
  drawView=5;
  drawSphere=YES;
  [self display];
  drawSphere=NO;
	needsClearing = YES;
return self;
}

- showAxSphere:sender {
  drawView=5;
  drawAxSphere=YES;
  [self display];
  drawAxSphere=NO;
	needsClearing = YES;
return self;
}

- showEuSphere:sender {
  drawView=5;
  drawEuSphere=YES;
  [self display];
  drawEuSphere=NO;
	needsClearing = YES;
return self;
}

- showShadows:sender {
  drawView=0;
  drawShadows=YES;
  [self display];
  drawShadows=NO;
	needsClearing = YES;
return self;
}

- iterCutting:sender
{int i; double gngl[5];
  if (strcmp([[cuttingMenu selectedCell] title],"orthogonal")==0)  
    for (i=0; i<5; i++) gngl[i]=90.0;
  if (strcmp([[cuttingMenu selectedCell] title],"all internal")==0)  
    for (i=0; i<5; i++) gngl[i]=45.0;
  if (strcmp([[cuttingMenu selectedCell] title],"3 int, 1 ext")==0)  
    {for (i=0; i<4; i++) gngl[i]=45.0; gngl[4]=180.0;}
  if (strcmp([[cuttingMenu selectedCell] title],"half & half")==0)  
    {for (i=0; i<3; i++) gngl[i]=45.0; gngl[3]=180.0; gngl[4]=180.0;}
  if (strcmp([[cuttingMenu selectedCell] title],"3 ext, 1 int")==0)  
    {gngl[0]=45.0; for (i=1; i<5; i++) gngl[i]=180.0;}
  if (strcmp([[cuttingMenu selectedCell] title],"all external")==0)  
    for (i=0; i<5; i++) gngl[i]=180.0;
  if (strcmp([[cuttingMenu selectedCell] title],"individual")==0)  
    for (i=0; i<5; i++) gngl[i]=qang[i];
  if (strcmp([[cuttingMenu selectedCell] title],"sphere 5")==0)  
    for (i=0; i<5; i++) gngl[i]=qang[4];
  fifth(qs,qs[4][3],gngl);
  georr(qt,qs,5);
  [[cutData cellAt:0:0] setDoubleValue:qt[4][0]];
  [[cutData cellAt:1:0] setDoubleValue:qt[4][1]];
  [[cutData cellAt:2:0] setDoubleValue:qt[4][2]];
  [[cutData cellAt:3:0] setDoubleValue:qt[4][3]];

return self;
}

- showCutting:sender {
  if (strcmp([[sender selectedCell] title],"four spheres")==0)  multiView=0;
  if (strcmp([[sender selectedCell] title],"fifth sphere")==0)  multiView=1;
  if (strcmp([[sender selectedCell] title],"five spheres")==0)  multiView=2;
  drawCutting=YES;
  [self display];
  drawCutting=NO;
	needsClearing = YES;
return self;
}

- printCutting:sender {
  multiView=2;
  drawCutting=YES;
  [self printPSCode:self];
  drawCutting=NO;
	needsClearing = YES;
return self;
}

- randCutting:sender {
  if (strcmp([[cutMenu selectedCell] title],"vertices")==0) thedr(qt);
  if (strcmp([[cutMenu selectedCell] title],"vertices")==0) qt[4][3]=0.2;
  if (strcmp([[cutMenu selectedCell] title],"space")==0) molra(qt,5);
  if (strcmp([[cutMenu selectedCell] title],"regular")==0) thon(qt);
  geoqr(qs,qt,5);
//  twopf(qs,5);
  drawCutting=YES;
  multiView=0;
  [self display];
  drawCutting=NO;
	needsClearing = YES;
return self;
}

- setAngleC:sender {
  [cutAngle setDoubleValue:90.0];
  [cutSlider setDoubleValue:90.0];
return self;
}

- setAngleR:sender {
  [cutAngle setDoubleValue:180.0];
  [cutSlider setDoubleValue:180.0];
return self;
}

- setAngleL:sender {
  [cutAngle setDoubleValue:0.0];
  [cutSlider setDoubleValue:0.0];
return self;
}

- randReflect:sender {int i;
  i=0;
  if (strcmp([sphereMenu title],"sphere 1")==0) i=0;
  if (strcmp([sphereMenu title],"sphere 2")==0) i=1;
  if (strcmp([sphereMenu title],"sphere 3")==0) i=2;
molran(rf,i);
return self;
}

- randReflectAll:sender {molra(rf,3); return self;}

- reflectInCentral:sender {int i; double f;
sphcf(rf[5],rf[0],3);
for (i=0; i<3; i++) {
  f=rf[i][0]*rf[i][0]+rf[i][1]*rf[i][1]+rf[i][2]*rf[i][2]-rf[i][3]*rf[i][3];
  rf[i][0]/=f;
  rf[i][1]/=f;
  rf[i][2]/=f;
  rf[i][3]=1.0/f;
  f=rf[i][0]*rf[i][0]+rf[i][1]*rf[i][1]+rf[i][2]*rf[i][2]-rf[i][3];
  rf[i][3]=f>0.0?sqrt(f):0.25;
  }
return self;
}

- showReflect:sender {
  drawReflect=YES;
  [self display];
  drawReflect=NO;
	needsClearing = YES;
return self;
}

- repeatReflect:sender {int i;
  drawReflect=YES;
  if ([repRef state]) for (i=0; i<10; i++) [self display];
  drawReflect=NO;
	needsClearing = YES;
return self;
}

- printReflect:sender {
  drawReflect=YES;
  [self printPSCode:self];
  drawReflect=NO;
	needsClearing = YES;
return self;
}

- showSplit:sender {
  drawSplit=YES;
  [self display];
  drawSplit=NO;
	needsClearing = YES;
return self;
}

- dismissPanels:sender {
    [gridPanel performClose:self];
    [liPanel performClose:self];
    [spPanel performClose:self];
    [frPanel performClose:self];
return self;
}

- prepSpiral:sender {
  pushMe=spImButton;
  [spImButton setTitle:"SPIRAL"];
  [self dismissPanels:self];
  if ([orientButton state]) [euPanel orderFront:self];
  if ([paramButton state]) [spPanel orderFront:self];
return self;
}

- prepFractal:sender {
  pushMe=spImButton;
  [spImButton setTitle:"FRACTAL"];
  [self dismissPanels:self];
  if ([orientButton state]) [euPanel orderFront:self];
  if ([paramButton state]) [frPanel orderFront:self];
return self;
}

- prepGrid:sender {
  pushMe=spImButton;
  [spImButton setTitle:"GRID"];
  [self dismissPanels:self];
  if ([orientButton state]) [euPanel orderFront:self];
  if ([paramButton state]) [gridPanel orderFront:self];
return self;
}

- prepLissa:sender {
  pushMe=spImButton;
  [spImButton setTitle:"LISSAJOUS"];
  [self dismissPanels:self];
  if ([orientButton state]) [euPanel orderFront:self];
  if ([paramButton state]) [liPanel orderFront:self];
return self;
}

- showUnisphere:sender {
  drawUnisphere=YES;
  [self display];
  drawUnisphere=NO;
	needsClearing = YES;
return self;
}

- showDisphere:sender {
int i;
double o[3][3], a[3], aa;
  if (strcmp([[rotateDimer selectedCell] title],"forward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(o,aa,a[0],a[1],a[2]);

     for (i=0; i<3; i++) a[i]=[[leftCenter cellAt:i:0] doubleValue];
     sphmv(a,o,a);
     for (i=0; i<3; i++) [[leftCenter cellAt:i:0] setDoubleValue:a[i]];

     for (i=0; i<3; i++) a[i]=[[rightCenter cellAt:i:0] doubleValue];
     sphmv(a,o,a);
     for (i=0; i<3; i++) [[rightCenter cellAt:i:0] setDoubleValue:a[i]];
     }
   if (strcmp([[rotateDimer selectedCell] title],"backward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(o,-aa,a[0],a[1],a[2]);

     for (i=0; i<3; i++) a[i]=[[leftCenter cellAt:i:0] doubleValue];
     sphmv(a,o,a);
     for (i=0; i<3; i++) [[leftCenter cellAt:i:0] setDoubleValue:a[i]];

     for (i=0; i<3; i++) a[i]=[[rightCenter cellAt:i:0] doubleValue];
     sphmv(a,o,a);
     for (i=0; i<3; i++) [[rightCenter cellAt:i:0] setDoubleValue:a[i]];
     }
 drawDisphere=YES;
  [self display];
  drawDisphere=NO;
	needsClearing = YES;
return self;
}

- printSphere:sender {
  drawView=5;
  drawSphere=YES;
  [self printPSCode:self];
  drawSphere=NO;
	needsClearing = YES;
return self;
}

- printUnisphere:sender {
  drawView=5;
  drawUnisphere=YES;
  [self printPSCode:self];
  drawUnisphere=NO;
	needsClearing = YES;
return self;
}

- printSpIm:sender {
  sslen=[[spiralData cellAt:1:0] intValue];
  sspch=[[spiralData cellAt:0:0] intValue];
  drawUnisphere=YES;
  [self printPSCode:self];
  drawUnisphere=NO;
	needsClearing = YES;

return self;
}

- printDisphere:sender {
  drawView=5;
  drawDisphere=YES;
  [self printPSCode:self];
  drawDisphere=NO;
	needsClearing = YES;

return self;
}

- rotateDimerB:sender {[dimerButton setTitle:"Backward"]; return self;}

- rotateDimerF:sender {[dimerButton setTitle:"Forward"]; return self;}

- rotateDimerN:sender {[dimerButton setTitle:"Dimer"]; return self;}

- pushIt:sender {[pushMe performClick:self]; return self;}

-qMom:sender {int i, j;
  if (strcmp([demoMenu title],"benzene")==0) geoqm(qpole,be,12);
  if (strcmp([demoMenu title],"sulfur")==0) geoqm(qpole,su,8);
  if (strcmp([demoMenu title],"ferrocene")==0) geoqm(qpole,fc,21);
  if (strcmp([demoMenu title],"polymer")==0) geoqm(qpole,mp,17);
  if (strcmp([demoMenu title],"random")==0) geoqm(qpole,rp,27);
  sphum(qdiag);
  sphcl(qpole,1.0e-3);
  for (i=0; i<3; i++) for (j=0; j<3; j++) {
    [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
    [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
    [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
    }
return self;
}

- dLize:header {int i, j;
      sphji(qdiag,qpole);
  sphcl(qpole,1.0e-4);
  for (i=0; i<3; i++) for (j=0; j<3; j++) {
    [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
    [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
    }
return self;
}

- inspectorPanel:sender {
  if (strcmp([demoMenu title],"benzene")==0) 
	[inspectBenzene makeKeyAndOrderFront:sender];
  if (strcmp([demoMenu title],"ferrocene")==0) 
	[inspectFerrocene makeKeyAndOrderFront:sender];
  if (strcmp([demoMenu title],"sulfur")==0) 
	[inspectSulfur makeKeyAndOrderFront:sender];
  if (strcmp([demoMenu title],"polymer")==0) 
	[inspectPolymer makeKeyAndOrderFront:sender];

return self;
}

- matrixPanel:sender {
  if (strcmp([demoMenu title],"polymer")==0) 
	[polymerMatrix makeKeyAndOrderFront:sender];
  drawSulfMatrix=YES;
  [polymerMatrix display];
  drawSulfMatrix=NO;

return self;
}

- inspectMolecule:sender {
  drawSulfMatrix=YES;
  [self display];
  drawSulfMatrix=NO;

return self;
}

- sliderOne:sender {int j; double p;
  p=0.0;
  j=[inspectorMenu selectedRow];
  if (strcmp([demoMenu title],"benzene")==0)   p=bep[j];
  if (strcmp([demoMenu title],"sulfur")==0)    p=sup[j];
  if (strcmp([demoMenu title],"ferrocene")==0) p=fcp[j];
  if (strcmp([demoMenu title],"polymer")==0)   p=mpp[j];
  [inspectorText setDoubleValue: p];
  [inspectorSlider setDoubleValue: 0.5];
  return self;
}

- sulfOne:sender {int j;
  j=[sulfMenu selectedRow];
  [sulfText setDoubleValue: supl[j]];
  [sulfSlider setDoubleValue: 0.5];
  return self;
}

- setOrigin:sender {return self;}

- setDistance: sender {int j; double p;

  j=[inspectorMenu selectedRow];
  p=[inspectorText doubleValue];

  if (strcmp([demoMenu title],"benzene")==0)   bep[j]=p;
  if (strcmp([demoMenu title],"sulfur")==0)    sup[j]=p;
  if (strcmp([demoMenu title],"ferrocene")==0) fcp[j]=p;
  if (strcmp([demoMenu title],"polymer")==0)   mpp[j]=p;

  [inspectorSlider setDoubleValue: 0.5];

  return self;
}

- showContour:sender {
  drawContour=YES;
  drawView=5;
  [self display];
  drawContour=NO;
return self;
}

/* - - - - - - - - - - - - - -    T R I D L  - - - - - - - - - - - - - - - - */

- setDynTridMat:sender {int i, j, k, l, m; double p, pa[10];
  k=[sulfMenu selectedRow];
  l=[sulfIndex selectedCol];
  m=[sulfAtom selectedRow];
  p=2.0*[sulfSlider doubleValue];
    for (i=0; i<8; i++) pa[i]=trpl[i];
    pa[k]=p-1.0;
    [sulfText setDoubleValue: pa[k]];
  for (i=0; i<TDB; i++) [[triMatrix cellAt :0 :i] setDoubleValue :pa[i]];
  trizdm(trdm);
  tridb(trdb,pa[0]);
  for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) {
    k=j-i+(TDB-1)/2;
    if (k>=0 && k<TDB) trdm[i][j]=trdb[i][k];
    }
  for (i=0; i<TDB; i++) [[triMatrix cellAt:0:i] setDoubleValue:trdb[m][i]];
  for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) trdi[i][j]=trdm[i][j];

  drawTriMatrix=YES;
  [self display];
  drawTriMatrix=NO;

return self;
}

- setTriPar: sender {int j; double p;
  j=[sulfMenu selectedRow];
  p=[sulfText doubleValue];
  trpl[j]=p;
  [inspectorSlider setDoubleValue: 0.5];
return self;
}

- gTri:sender {
  if (PDM==11) elvji(trnm,trdm); else printf("wrong dimension\b");
  drawTriMatrix=YES;
  [self display];
  drawTriMatrix=NO;
return self;
}

- showNormalTridMode:sender {
  drawTriMode=YES;
  drawView=5;
  [self display];
  drawTriMode=NO;
return self;
}

- showTri:sender {
  drawTriMatrix=YES;
  [self display];
  drawTriMatrix=NO;
return self;
}

- tryTriDemo:sender {int i, j;
//  for (i=0; i<6; i++) for (j=0; j<11; j++) trdm[i][j]=2.0*trdi[i][j];
//  for (i=6; i<11; i++) for (j=0; j<11; j++) trdm[i][j]=trdi[i][j];
//  trdm[5][6]*=0.7;
//  trdm[6][5]*=0.7;

//  for (i=0; i<11; i++) for (j=0; j<11; j++) trdm[i][j]=trdi[i][j];
//  for (i=0; i<11; i+=2) trdm[i][i]=2.0*trdi[i][i];

//  for (i=0; i<11; i++) for (j=0; j<11; j++) trdm[i][j]=trdi[i][j];
//  trdm[4][5]*=2.0; trdm[5][4]*=2.0;
//  trdm[5][5]*=4.0;
//  trdm[5][6]*=2.0; trdm[6][5]*=2.0;

  for (i=0; i<11; i++) for (j=0; j<11; j++) trdm[i][j]=trdi[i][j];
  for (i=0; i<11; i++) trdm[i][i]/=(1.0+0.1*(double)(i));
  for (i=0; i<10; i++) trdm[i][i+1]/=(1.0+0.1*(double)(i));
  for (i=1; i<11; i++) trdm[i-1][i] = trdm[i][i-1];

return self;
}

/* - - - - - - - - - - - - - -    P E N T A  - - - - - - - - - - - - - - - - */

- setDynPentMat:sender {int i, j, k, l, m; double p, pa[10];

  k=[sulfMenu selectedRow];
  l=[sulfIndex selectedCol];
  m=[sulfAtom selectedRow];
  p=2.0*[sulfSlider doubleValue];
    for (i=0; i<8; i++) pa[i]=pepl[i];
    pa[k]=p-1.0;
    [sulfText setDoubleValue: pa[k]];
    for (i=0; i<PDB; i++) [[penMatrix cellAt :0 :i] setDoubleValue :pa[i]];
penzdm(pedy);
pendb(pedb,pa[0],pa[1]);
for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) {
  k=j-i+(PDB-1)/2;
  if (k>=0 && k<PDB) pedy[i][j]=pedb[i][k];
  }
    for (i=0; i<PDB; i++) [[penMatrix cellAt:0:i] setDoubleValue:pedb[m][i]];
for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) pedi[i][j]=pedy[i][j];

  drawPenMatrix=YES;
  [self display];
  drawPenMatrix=NO;

return self;
}

- setPenPar: sender {int j; double p;
  j=[sulfMenu selectedRow];
  p=[sulfText doubleValue];
  pepl[j]=p;
  [inspectorSlider setDoubleValue: 0.5];
return self;
}

- gPen:sender {
  if (PDM==11) elvji(penm,pedy); else printf("wrong dimension\b");
  drawPenMatrix=YES;
  [self display];
  drawPenMatrix=NO;
return self;
}

- showNormalPentMode:sender {
  drawPenMode=YES;
  drawView=5;
  [self display];
  drawPenMode=NO;
return self;
}

- showPen:sender {
  drawPenMatrix=YES;
  [self display];
  drawPenMatrix=NO;

return self;
}

/* - - - - - - - - - - - - - - -   S U L F   - - - - - - - - - - - - - - - - */

- zsulf:sender {supl[[sulfMenu selectedRow]]=0.0; return self;}

- usulf:sender {supl[[sulfMenu selectedRow]]=1.0; return self;}

- showMatrix:sender {int i, j, l, m;
  l=[sulfIndex selectedCol];
  m=[sulfAtom selectedRow];
for (i=0; i<3; i++) for (j=0; j<3; j++) 
[[sulfMatrix cellAt:i:j] setDoubleValue:sudb[m][l][i][j]];
return self;
}

- runNormalSulfMode:sender {
  runCycle=1;
  if (strcmp([[sulfRun selectedCell] title],"1")==0) runCycle=1;
  if (strcmp([[sulfRun selectedCell] title],"50")==0) runCycle=50;
  if (strcmp([[sulfRun selectedCell] title],"500")==0) runCycle=500;

  drawSulfMode=YES;
  drawView=5;
  [self display];
  drawSulfMode=NO;
return self;
}

- showNormalSulfMode:sender {
  runCycle=1;
  drawSulfMode=YES;
  drawView=5;
  [self display];
  drawSulfMode=NO;
return self;
}

- printMatrix:sender {
  runCycle=1;
  drawSulfMode=YES;
  drawView=5;
  [self printPSCode:self];
  drawSulfMode=NO;
return self;
}

- setDynSulfMat:sender
{int i, j, k, l, m; double p, pa[10], w[3][3];

  k=[sulfMenu selectedRow];
  l=[sulfIndex selectedCol];
  m=[sulfAtom selectedRow];

  p=2.0*[sulfSlider doubleValue];

    for (i=0; i<8; i++) pa[i]=supl[i];
    pa[k]=p-1.0;
    [sulfText setDoubleValue: pa[k]];

sulsq(suls,mp,17);
for (i=0; i<11; i++) suldb(sudb,suls[3],pa,i);
[self showMatrix:self];
sulzm(w);
for (i=0; i<7; i++) sulau(w,w,1.0,sudb[m][i]);
//for (i=0; i<3; i++) for (j=0; j<3; j++) printf("%6.3f ",sudb[m][l][i][j]);
//printf("\n");
p=0.0;
for (i=0; i<3; i++) for (j=0; j<3; j++) p+=w[i][j]*w[i][j];
[sulfError setDoubleValue:sqrt(p)];

for (i=0; i<11; i++) for (j=0; j<11; j++) sulzm(sudy[i][j]);
for (i=0; i<11; i++) for (j=0; j<11; j++) {
  k=j-i+3;
  if (k>=0 && k<7) sulcm(sudy[i][j],sudb[i][k]);
  }
// sulcd(sudy,sudb,m,11);
//printf("dynamical matrix:00 %6.3f, 11 %6.3f, 22 %6.3f, 01 %6.3f, 10 %6.3f\n\n",
//	sudy[0][0][0][0],sudy[0][0][1][1],sudy[0][0][2][2],
//	sudy[0][0][0][1],sudy[0][0][1][0]);

for (i=0; i<11; i++) for (j=0; j<11; j++) sulcm(sudi[i][j],sudy[i][j]);

  [diagButton setEnabled:YES];

  drawSulfMatrix=YES;
  [self display];
  drawSulfMatrix=NO;

return self;
}

- gSulf:sender {
jbyji(sunm,sudy);
//printf("diagonal dynamical matrix:00 %6.3f, 11 %6.3f, 22 %6.3f,
//	 01 %6.3f, 10 %6.3f\n\n",
//	sudy[0][0][0][0],sudy[0][0][1][1],sudy[0][0][2][2],
//	sudy[0][0][0][1],sudy[0][0][1][0]);
//printf("eigenvector components:    00 %6.3f, 11 %6.3f, 22 %6.3f,
//	 01 %6.3f, 10 %6.3f\n\n",
//	sunm[0][0][0][0],sunm[0][0][1][1],sunm[0][0][2][2],
//	sunm[0][0][0][1],sunm[0][0][1][0]);

  [diagButton setEnabled:NO];

  drawSulfMatrix=YES;
  [self display];
  drawSulfMatrix=NO;

return self;
}

- setSulPar: sender {int j; double p;
  j=[sulfMenu selectedRow];
  p=[sulfText doubleValue];
  supl[j]=p;
  [inspectorSlider setDoubleValue: 0.5];
return self;
}

/* - - - - - - - - - - - - -  G E O M  V I E W   - - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new GeomView object.  First, an initFrame: message is sent 
 * to super to initialize GeomView as a View.  Next, the GeomView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in  the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
	[self setOpaque:YES];
	[self setRadius:DEFAULTRADIUS];
	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)];
	points = [[Storage alloc] 
			initCount:0 
			elementSize:sizeof(NXPoint) 
			description:"{ff}"];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];
	/* 
	 * Normally, the next two message expressions would be combined, as:
	 * 		readOut = [[Cell alloc] initTextCell:""];
	 * but are here separated for clarity.
	 */
	readOut = [Cell alloc];
	[readOut initTextCell:""];
	[readOut setBezeled:YES];
	loadPSProcedures();
	return self;
}


- setDelegate:anObject
/*
 * Sets the GeomView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the GeomView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although GeomView only repaints the background 
 * of the update region, it redraws the entire axes. For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	unsigned int	 i, j, k, l, m, n, v;
	int              i0, i1, i2;
	NXPoint		 *aPoint;
        double           p, si, aa, a[3], w[2], z[2], pa[10];
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context. This simple test is OK for a View that prints a 
	 * single page.  If there were multiple pages, the procedures would be 
	 * loaded multiple times.  Although this is inefficient, it causes no 
	 * error.  Views that print multiple pages should load procedures such 
	 * as these in an endPrologue method (see the View spec sheet). 
	 */
	if (NXDrawingStatus != NX_DRAWING) 
		loadPSProcedures();

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);
	if (drawBack) drawAxes(
		bounds.origin.x,
		bounds.origin.y,
		bounds.size.width,
		bounds.size.height
		);

	/* now take each point and draw it */
	PSsetgray(NX_BLACK);
	i = [points count];
	while (i--) {
		aPoint = (NXPoint *)[points elementAt:i];
		drawCircle(aPoint->x, aPoint->y, radius);
	} 


     if (drawAxSphere) {
	[self rotateMolecule:self];
	sphgeu(a0,mrot);
	sphzv(r0);
        PSnewpath();
        geomss(r0,0.75,a0,7,150);
	PSmoveto(0.0,0.0);
	PSarc(0.0,0.0,50.0*0.75,0.0,360.0);
	PSstroke();
	}

else if (drawEuSphere) {
	[self readEulerAngles:self];
	sphzv(r0);
        geomss(r0,0.75,a0,12,180);
	PSmoveto(0.0,0.0);
	PSarc(0.0,0.0,50.0*0.75,0.0,360.0);
	PSstroke();
	}

else if (drawEditedMolecule) {

  PSnewpath();

  if (strcmp([viewMenu title],"front")==0) v=0;
  if (strcmp([viewMenu title],"side")==0)  v=1;
  if (strcmp([viewMenu title],"plan")==0)  v=2;

  p=[radiusSlider doubleValue];

  if (strcmp([demoMenu title],"benzene")==0) {
    for (i=0; i<12; i++) fsipc(be[i],p*be[i][3],v);
    }
  if (strcmp([demoMenu title],"sulfur")==0) {
    for (i=0; i<8; i++) fsipc(su[i],p*su[i][3],v);
    }
  if (strcmp([demoMenu title],"ferrocene")==0) {
    for (i=0; i<21; i++) fsipc(fc[i],p*fc[i][3],v);
    }
  if (strcmp([demoMenu title],"polymer")==0) {
    for (i=0; i<17; i++) fsipc(mp[i],p*mp[i][3],v);
    }

  if (drawView==5) PSstroke();

  }

else if (drawTriMatrix) { 

  if (strcmp([[sulfShow selectedCell] title],"H")==0) 
	 for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) {
       if (trdi[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (-0.1<trdi[i][j]&&trdi[i][j]<0.1) PSsetgray(NX_DKGRAY);
  else if (-0.5<trdi[i][j]&&trdi[i][j]<0.5) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,23.0,23.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"L")==0) {
	 for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) {
       if (trdm[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (-0.01<trdm[i][j]&&trdm[i][j]<0.01) PSsetgray(NX_DKGRAY);
  else if (-0.05<trdm[i][j]&&trdm[i][j]<0.05) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }
	printf("\n");
	for (i=0; i<TDM; i++) 
		printf("%2d, %8.5f\n",i,trdm[i][i]);
	printf("\n");
  }

  if (strcmp([[sulfShow selectedCell] title],"U")==0) 
	 for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) {
       if (trnm[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (trnm[i][j]<0.0) PSsetgray(NX_DKGRAY);
  else if (trnm[i][j]>0.0) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"B")==0) 
	 for (i=0; i<TDM; i++) for (j=0; j<PDB; j++) {
   if(trdb[i][j]==0.0) PSsetgray(NX_BLACK); else PSsetgray(NX_WHITE);
   PSrectfill(12.0*(float)j-130.0,12.0*(float)i,4.0,4.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"s")==0) 
	 for (i=0; i<TDM; i++) for (j=0; j<TDM; j++) {
           p=trdi[i][j]-trdi[j][i];
	   p=(p<0.0?-p:p);
                if (p<0.001) PSsetgray(NX_BLACK);
           else if (p<0.1)   PSsetgray(NX_LTGRAY);
           else              PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"e")==0) {
    for (i=0; i<TDM; i++) {
      pltms(0.5*trdm[i][i],0.0,0);
      pltms(0.5*trdm[i][i],1.0,1);
      }
    if (drawView==5) PSstroke();
    }
}

else if (drawPenMatrix) { 

  if (strcmp([[sulfShow selectedCell] title],"H")==0) 
	 for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) {
       if (pedi[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (-0.1<pedi[i][j]&&pedi[i][j]<0.1) PSsetgray(NX_DKGRAY);
  else if (-0.5<pedi[i][j]&&pedi[i][j]<0.5) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,23.0,23.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"L")==0) {
	 for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) {
       if (pedy[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (-0.01<pedy[i][j]&&pedy[i][j]<0.01) PSsetgray(NX_DKGRAY);
  else if (-0.05<pedy[i][j]&&pedy[i][j]<0.05) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }
	printf("\n");
	for (i=0; i<PDM; i++) 
		printf("%2d, %8.5f\n",i,pedy[i][i]);
	printf("\n");
  }

  if (strcmp([[sulfShow selectedCell] title],"U")==0) 
	 for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) {
       if (penm[i][j]==0.0) PSsetgray(NX_BLACK);
  else if (penm[i][j]<0.0) PSsetgray(NX_DKGRAY);
  else if (penm[i][j]>0.0) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"B")==0) 
	 for (i=0; i<PDM; i++) for (j=0; j<PDB; j++) {
   if(pedb[i][j]==0.0) PSsetgray(NX_BLACK); else PSsetgray(NX_WHITE);
   PSrectfill(12.0*(float)j-130.0,12.0*(float)i,4.0,4.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"s")==0) 
	 for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) {
           p=pedi[i][j]-pedi[j][i];
	   p=(p<0.0?-p:p);
                if (p<0.001) PSsetgray(NX_BLACK);
           else if (p<0.1)   PSsetgray(NX_LTGRAY);
           else              PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j-130.0,110.0-24.0*(float)i,20.0,20.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"e")==0) {
    for (i=0; i<PDM; i++) {
      pltms(0.5*pedy[i][i],0.0,0);
      pltms(0.5*pedy[i][i],1.0,1);
      }
    if (drawView==5) PSstroke();
    }
}

else if (drawSulfMatrix) {

  if (strcmp([[sulfShow selectedCell] title],"H")==0) 
	 for (i=0; i<11; i++) for (j=0; j<11; j++) 
 	 for (k=0; k<3; k++) for (l=0; l<3; l++) {
       if (sudi[i][j][k][l]==0.0) PSsetgray(NX_BLACK);
  else if (-0.1<sudi[i][j][k][l]&&sudi[i][j][k][l]<0.1) PSsetgray(NX_DKGRAY);
  else if (-0.5<sudi[i][j][k][l]&&sudi[i][j][k][l]<0.5) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j+8.0*(float)l-130.0,
	110.0-24.0*(float)i+8.0*(float)(2-k),8.0,8.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"L")==0) {
	 for (i=0; i<11; i++) for (j=0; j<11; j++) 
 	 for (k=0; k<3; k++)  for (l=0; l<3; l++) {
       if (sudy[i][j][k][l]==0.0) PSsetgray(NX_BLACK);
  else if (-0.01<sudy[i][j][k][l]&&sudy[i][j][k][l]<0.01) PSsetgray(NX_DKGRAY);
  else if (-0.05<sudy[i][j][k][l]&&sudy[i][j][k][l]<0.05) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j+8.0*(float)l-130.0,
	110.0-24.0*(float)i+8.0*(float)(2-k),8.0,8.0);
  }
//	printf("\n");
//	for (i=0; i<11; i++) for (j=0; j<3; j++)
//		printf("%2d, %2d, %8.5f\n",i,j,sudy[i][j][i][j]);
//	printf("\n");
  }

  if (strcmp([[sulfShow selectedCell] title],"U")==0) 
	 for (i=0; i<11; i++) for (j=0; j<11; j++) 
 	 for (k=0; k<3; k++) for (l=0; l<3; l++) {
       if (sunm[i][j][k][l]==0.0) PSsetgray(NX_BLACK);
  else if (sunm[i][j][k][l]<0.0) PSsetgray(NX_DKGRAY);
  else if (sunm[i][j][k][l]>0.0) PSsetgray(NX_LTGRAY);
  else    PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j+8.0*(float)l-130.0,
	110.0-24.0*(float)i+8.0*(float)(2-k),8.0,8.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"B")==0) 
	 for (i=0; i<11; i++) for (j=0; j<7; j++) 
 	 for (k=0; k<3; k++) for (l=0; l<3; l++) {
   if(sudb[i][j][k][l]==0.0) PSsetgray(NX_BLACK); else PSsetgray(NX_WHITE);
   PSrectfill(12.0*(float)j+4.0*(float)l,12.0*(float)i+4.0*(float)k,4.0,4.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"s")==0) 
	 for (i=0; i<11; i++) for (j=0; j<11; j++) 
 	 for (k=0; k<3; k++) for (l=0; l<3; l++) {
           p=sudi[i][j][k][l]-sudi[j][i][l][k];
	   p=(p<0.0?-p:p);
                if (p<0.01) PSsetgray(NX_BLACK);
           else if (p<0.1)  PSsetgray(NX_LTGRAY);
           else             PSsetgray(NX_WHITE);
  PSrectfill(24.0*(float)j+8.0*(float)l-130.0,
	110.0-24.0*(float)i+8.0*(float)(2-k),8.0,8.0);
  }

  if (strcmp([[sulfShow selectedCell] title],"e")==0) {
    for (i=0; i<11; i++) for (j=0; j<3; j++) {
      pltms(3.0*sudy[i][j][i][j],0.0,0);
      pltms(3.0*sudy[i][j][i][j],1.0,1);
      }
    if (drawView==5) PSstroke();
    }
}

else if (drawContour) {

  if (strcmp([viewMenu title],"front")==0) v=0;
  if (strcmp([viewMenu title],"side")==0)  v=1;
  if (strcmp([viewMenu title],"plan")==0)  v=2;

  if (strcmp([[contourMenu selectedCell] title],"sphere")==0) i=0;
  if (strcmp([[contourMenu selectedCell] title],"plane")==0)  i=1;
  if (strcmp([[contourMenu selectedCell] title],"1-sheet h")==0)  i=2;
  if (strcmp([[contourMenu selectedCell] title],"torus")==0)  i=3;
  if (strcmp([[contourMenu selectedCell] title],"bitorus")==0)  i=4;
  if (strcmp([[contourMenu selectedCell] title],"2-sheet h")==0)  i=5;
  if (strcmp([[contourMenu selectedCell] title],"cubic")==0)  i=6;
  if (strcmp([[contourMenu selectedCell] title],"ellipsoid")==0)  i=7;
      cudem(cubic,25,25,25,i);
    if (i==4) p=0.0; else p=[sulfSlider doubleValue];
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++)
      conth(cubic,p,i,j,k,v);

    if (drawView==5) PSstroke();

}

else if (drawFatMolecule) {

  if (strcmp([viewMenu title],"front")==0) v=0;
  if (strcmp([viewMenu title],"side")==0)  v=1;
  if (strcmp([viewMenu title],"plan")==0)  v=2;

  if (strcmp([demoMenu title],"benzene")==0)   fsisf(be,12,v);
  if (strcmp([demoMenu title],"sulfur")==0)    fsisf(su,8,v);
  if (strcmp([demoMenu title],"ferrocene")==0) fsisf(fc,21,v);
  if (strcmp([demoMenu title],"polymer")==0)   fsisf(mp,17,v);

  if (drawView==5) PSstroke();

}

else if (drawInspectedMolecule) {

  if (strcmp([viewMenu title],"front")==0) v=0;
  if (strcmp([viewMenu title],"side")==0)  v=1;
  if (strcmp([viewMenu title],"plan")==0)  v=2;

  j=[inspectorMenu selectedRow];

  p=2.0*[inspectorSlider doubleValue];

  if (strcmp([demoMenu title],"benzene")==0) {
    for (i=0; i<4; i++) pa[i]=bep[i];
    pa[j]=p*bep[j];
    [inspectorText setDoubleValue: pa[j]];
    molbe(be,pa[2],pa[1],pa[3],pa[0]+pa[1]);    
    for (i=0; i<12; i++) fsipc(be[i],be[i][3],v);
    }
  if (strcmp([demoMenu title],"sulfur")==0) {
    for (i=0; i<2; i++) pa[i]=sup[i];
    pa[j]=p*sup[j];
    [inspectorText setDoubleValue: pa[j]];
    molsu(su,pa[0],pa[1],8);    
    for (i=0; i<8; i++) fsipc(su[i],su[i][3],v);
    }
  if (strcmp([demoMenu title],"ferrocene")==0) {
    for (i=0; i<7; i++) pa[i]=fcp[i];
    pa[j]=p*fcp[j];
    [inspectorText setDoubleValue: pa[j]];
    molfc(fc,pa[5],pa[3],pa[4],pa[0]);    
    for (i=0; i<21; i++) fsipc(fc[i],0.5*fc[i][3],v);
    }
  if (strcmp([demoMenu title],"polymer")==0) {
    for (i=0; i<6; i++) pa[i]=mpp[i];
    pa[j]=p*mpp[j];
    [inspectorText setDoubleValue: pa[j]];
    molmp(mp,pa[0],pa[1],pa[2],pa[3],17);
    for (i=0; i<17; i++) fsipc(mp[i],0.5*mp[i][3],v);
    }

  if (drawView==5) PSstroke();

  }

else if (drawTriMode) {
  i=[fAtom selectedCol];
  [sulfText setDoubleValue:trdm[i][i]];
  pltms(-3.0,0.0,0);
  for (k=0; k<TDM; k++) {
     p=-2.5+0.5*(double)k;
     pltms(p,trnm[k][i],1);
    }
  pltms(3.0,0.0,1);
  if (drawView==5) PSstroke();
  for (k=0; k<TDM; k++) {
     p=-2.5+0.5*(double)k;
     drawCircle(50.0*p,50.0*trnm[k][i],2.5);
    }
  }

else if (drawPenMode) {
  i=[fAtom selectedCol];
  [sulfText setDoubleValue:pedy[i][i]];
  pltms(-3.0,0.0,0);
  for (k=0; k<PDM; k++) {
     p=-2.5+0.5*(double)k;
     pltms(p,penm[k][i],1);
    }
  pltms(3.0,0.0,1);
  if (drawView==5) PSstroke();
  for (k=0; k<PDM; k++) {
     p=-2.5+0.5*(double)k;
     drawCircle(50.0*p,50.0*penm[k][i],2.5);
    }
  }

else if (drawSulfMode) {

  i0=0; i1=1; i2=2;
  if (strcmp([viewMenu title],"front")==0) {i0=0; i1=1; i2=2;}
  if (strcmp([viewMenu title],"side")==0)  {i0=1; i1=0; i2=2;}
  if (strcmp([viewMenu title],"plan")==0)  {i0=0; i1=2; i2=1;}

  i=[fAtom selectedCol];
  j=[fCoord selectedCol];

  [sulfText setDoubleValue:sudy[i][j][i][j]];

  pltms(suls[3][i0],suls[3][i2],0);
for (k=0; k<11; k++) pltms(suls[k+3][i0],suls[k+3][i2],1);
  pltms(suls[3][i0],suls[3][i2],0);
for (k=0; k<11; k++) 
    pltms(suls[k+3][i0]+sunm[k][i][i0][j],
	suls[k+3][i2]+sunm[k][i][i2][j],1);
  pltms(suls[3][i0],suls[3][i2],0);
for (k=0; k<11; k++) 
  pltms(suls[k+3][i0]-sunm[k][i][i0][j],
	suls[k+3][i2]-sunm[k][i][i2][j],1);

  if (drawView==5) PSstroke();

if (runCycle>1) {
        PSsetinstance(YES);
  for (l=0, p=0.0; l<runCycle; l++, p+=0.1) {
    si=[sulfAmplitude doubleValue]*sin(p);
		PSnewinstance();
    pltms(suls[3][i0],suls[3][i2],0);
    for (k=0; k<11; k++) 
      pltms(suls[k+3][i0]+si*sunm[k][i][i0][j],
	suls[k+3][i2]+si*sunm[k][i][i2][j],1);
    if (drawView==5) PSstroke();
		[window flushWindow];
    for (k=0; k<11; k++) 
      drawCircle(50*(suls[k+3][i0]+si*sunm[k][i][i0][j]),
	50*(suls[k+3][i2]+si*sunm[k][i][i2][j]),2.5);
    if (drawView==5) PSstroke();
		[window flushWindow];
    }
  }

  }

else if (drawUnisphere) {

  [self readEulerAngles:self];
  spheu(mrot,a0);
  sphdc(a,mrot);
  [[angleParameters cellAt:0:0] setDoubleValue:a[0]]; 
  [[angleParameters cellAt:1:0] setDoubleValue:a[1]]; 
  [[angleParameters cellAt:2:0] setDoubleValue:a[2]]; 
  [[angleParameters cellAt:3:0] setDoubleValue:sphar(mrot)]; 
  spRadius=[radiusSlider doubleValue];
  if (strcmp([[primiMenu selectedCell] title],"points")==0) drawView=0;
  if (strcmp([[primiMenu selectedCell] title],"lines")==0) drawView=5;

  if (strcmp([[traceMenu selectedCell] title],"spiral")==0) {
     sslen=[[spiralData cellAt:1:0] intValue];
     sspch=[[spiralData cellAt:0:0] intValue];
     geomss(r0,spRadius,a0,sspch,sslen);
     }
  if (strcmp([[traceMenu selectedCell] title],"grid")==0) {
     nlat=[[gridData cellAt:0:0] intValue];
     nlong=[[gridData cellAt:1:0] intValue];
     clat=[[laloData cellAt:0:0] intValue];
     clong=[[laloData cellAt:1:0] intValue];
     geomsg(r0,spRadius,a0,nlong,nlat,clong,clat);
     }
  if (strcmp([[traceMenu selectedCell] title],"Lissajous")==0) {
     sllen=[[spiralData cellAt:2:0] intValue];
     nlat=[[lissaData cellAt:0:0] intValue];
     nlong=[[lissaData cellAt:1:0] intValue];
     clat=[[lissaData cellAt:2:0] intValue];
     geomsl(r0,spRadius,a0,nlong,nlat,clat);
     }
  if (strcmp([[traceMenu selectedCell] title],"fractal")==0) {
     frdepth=[[frData selectedCell] title][0]-'0';
     geofs(r0,spRadius,a0,frdepth);
     }

  if ([rimButton state]) {PSmoveto(0.0,0.0); PSarc(0.0,0.0,50.0*spRadius,0.0,360.0);}
  if (drawView==5) PSstroke();
  }

else if (drawDisphere) {
drawView=5;
s[0][0]=[[leftCenter cellAt:0:0] doubleValue];
s[0][1]=[[leftCenter cellAt:1:0] doubleValue];
s[0][2]=[[leftCenter cellAt:2:0] doubleValue];
s[0][3]=[leftRadius doubleValue];
s[0][4]=[[leftEuler cellAt:0:0] doubleValue];
s[0][5]=[[leftEuler cellAt:1:0] doubleValue];
s[0][6]=[[leftEuler cellAt:2:0] doubleValue];
spheu(o,&s[0][4]);
sphdc(a,o);
  [[leftAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[leftAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[leftAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[leftAngles cellAt:3:0] setDoubleValue:sphar(o)]; 
s[1][0]=[[rightCenter cellAt:0:0] doubleValue];
s[1][1]=[[rightCenter cellAt:1:0] doubleValue];
s[1][2]=[[rightCenter cellAt:2:0] doubleValue];
s[1][3]=[rightRadius doubleValue];
s[1][4]=[[rightEuler cellAt:0:0] doubleValue];
s[1][5]=[[rightEuler cellAt:1:0] doubleValue];
s[1][6]=[[rightEuler cellAt:2:0] doubleValue];
spheu(o,&s[1][4]);
sphdc(a,o);
  [[rightAngles cellAt:0:0] setDoubleValue:a[0]]; 
  [[rightAngles cellAt:1:0] setDoubleValue:a[1]]; 
  [[rightAngles cellAt:2:0] setDoubleValue:a[2]]; 
  [[rightAngles cellAt:3:0] setDoubleValue:sphar(o)]; 

geosi(s);
  if ([rimButton state]) geoci(s);
  if (drawView==5) PSstroke();
}

else if (drawShadows) {sspch=79; sslen=2048;
  if (strcmp([[primiMenu selectedCell] title],"points")==0) drawView=0;
  if (strcmp([[primiMenu selectedCell] title],"lines")==0) drawView=5;
 shadsf(u,tu,uu,6); if (drawView==5) PSstroke();}

else if (drawReflect) {
  if (strcmp([[primiMenu selectedCell] title],"points")==0) drawView=0;
  if (strcmp([[primiMenu selectedCell] title],"lines")==0) drawView=5;
  if (strcmp([[rotateDimer selectedCell] title],"forward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(mrot,aa,a[0],a[1],a[2]);
     georf(rf,mrot,6);
     }
  if (strcmp([[rotateDimer selectedCell] title],"backward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(mrot,-aa,a[0],a[1],a[2]);
     georf(rf,mrot,6);
     }
 eomsf(rf,6); if (drawView==5) PSstroke();}

else if (drawSplit) {
  if (strcmp([[primiMenu selectedCell] title],"points")==0) drawView=0;
  if (strcmp([[primiMenu selectedCell] title],"lines")==0) drawView=5;
//  if (strcmp([[rotateDimer selectedCell] title],"forward")==0) {
//     [self readAngleAxis:a:&aa];
//     sphax(mrot,aa,a[0],a[1],a[2]);
//     georf(rf,mrot,6);
//     }
//  if (strcmp([[rotateDimer selectedCell] title],"backward")==0) {
//     [self readAngleAxis:a:&aa];
//     sphax(mrot,-aa,a[0],a[1],a[2]);
//     georf(rf,mrot,6);
//     }
// eomsf(rf,6);
	sphzv(r0);
	[self readEulerAngles: self];
	fsiss(r0,1.75,a0,20,800); 
 if (drawView==5) PSstroke();}

else if (drawCutting) {sspch=32; sslen=512;
  if (strcmp([[primiMenu selectedCell] title],"points")==0) drawView=0;
  if (strcmp([[primiMenu selectedCell] title],"lines")==0) drawView=5;
  [[cutData cellAt:0:0] setDoubleValue:qt[4][0]];
  [[cutData cellAt:1:0] setDoubleValue:qt[4][1]];
  [[cutData cellAt:2:0] setDoubleValue:qt[4][2]];
  [[cutData cellAt:3:0] setDoubleValue:qt[4][3]];
  if (strcmp([[rotateDimer selectedCell] title],"forward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(mrot,aa,a[0],a[1],a[2]);
     georf(qt,mrot,5);
     }
  if (strcmp([[rotateDimer selectedCell] title],"backward")==0) {
     [self readAngleAxis:a:&aa];
     sphax(mrot,-aa,a[0],a[1],a[2]);
     georf(qt,mrot,5);
     }
  switch (multiView) {
    case 0: eomsf(qt,4); break;
    case 1: eomsf(qt[4],1); break;
    case 2: eomsf(qt,5); break;
    default: break;}
  if (drawView==5) PSstroke();}

else if (drawSphere) { 

  if (strcmp([viewMenu title],"front")==0) v=0;
  if (strcmp([viewMenu title],"side")==0)  v=1;
  if (strcmp([viewMenu title],"plan")==0)  v=2;

  geoii();
  sphdc(a,mrot);
  [[angleParameters cellAt:0:0] setDoubleValue:a[0]]; 
  [[angleParameters cellAt:1:0] setDoubleValue:a[1]]; 
  [[angleParameters cellAt:2:0] setDoubleValue:a[2]]; 
  [[angleParameters cellAt:3:0] setDoubleValue:sphar(mrot)]; 

  if (strcmp([demoMenu title],"sphere")==0) {
      if (strcmp([gridMenu title],"spiral")==0) geodi(1);
      if (strcmp([gridMenu title],"grid")==0) geodi(2);
      if (strcmp([gridMenu title],"lissajous")==0) geodi(3);
      if (strcmp([gridMenu title],"fractal")==0) geodi(11);
      }
  if (strcmp([demoMenu title],"geosi")==0) geosi(s);
  if (strcmp([demoMenu title],"eomsf")==0) fsisf(u,6,v);
  if (strcmp([demoMenu title],"shadow")==0) shadsf(u,tu,uu,6);

  if (strcmp([demoMenu title],"benzene")==0) {
      geoqm(qpole,be,12);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
	[quadPanel orderFront];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(be,mrot,12);
      georf(be,qdiag,12);
      if ([showNuclei state]) {
        geosr(be,0.20,12);
        fsisf(be,12,v);
        if ([showBonds state]) geobf(be,bbe,12);
        geosr(be,5.0,12);
        }
      if ([showAtoms state]) fsisf(be,12,v);
      if ([showAxes state]) geodax(be,mrot,12);
      }

  if (strcmp([demoMenu title],"sulfur")==0) {
      geoqm(qpole,su,8);
  sphcl(qpole,1.0e-3);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
	[quadPanel orderFront];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(su,mrot,8);
      georf(su,qdiag,8);
      if ([showNuclei state]) {
        geosr(su,0.20,8);
        fsisf(su,8,v);
//        if ([showBonds state]) geobf(be,bbe,12);
        geosr(su,5.0,8);
        }
      if ([showAtoms state]) fsisf(su,8,v);      
      if ([showAxes state]) geodax(su,mrot,8);
      }

  if (strcmp([demoMenu title],"ferrocene")==0) {
      geoqm(qpole,fc,21);
  sphcl(qpole,1.0e-3);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
	[quadPanel orderFront];
  sphcl(qpole,1.0e-3);
  sphcl(qdiag,1.0e-3);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(fc,mrot,21);
      georf(fc,qdiag,21);
      [self rotateMolecule:self];
      georf(fc,mrot,21);
      if ([showNuclei state]) {
        geosr(fc,0.20,21);
        fsisf(fc,21,v);
//        if ([showBonds state]) geobf(be,bbe,12);
        geosr(fc,5.0,21);
        }
      if ([showAtoms state]) fsisf(fc,21,v);
      if ([showAxes state]) geodax(fc,mrot,21);
      }

  if (strcmp([demoMenu title],"polymer")==0) {
      geoqm(qpole,mp,17);
  sphcl(qpole,1.0e-3);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
	[quadPanel orderFront];
  sphcl(qpole,1.0e-3);
  sphcl(qdiag,1.0e-3);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(mp,mrot,17);
//      georf(mp,qdiag,17);
//      [self rotateMolecule:self];
//      georf(mp,mrot,17);
//      if ([showNuclei state]) {
//        geosr(mp,0.20,17);
//        fsisf(mp,17,v);
// //       if ([showBonds state]) geomp(be,bbe,17);
//        geosr(mp,5.0,17);
//        }
      if ([showAtoms state]) fsisf(mp,17,v);
      if ([showAxes state]) geodax(mp,mrot,17);
      }

  if (strcmp([demoMenu title],"random")==0) {
      geoqm(qpole,rp,27);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
	[quadPanel orderFront];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(rp,mrot,27);
      georf(rp,qdiag,27);
      if ([showAtoms state]) fsisf(rp,27,v);      
      if ([showAxes state]) geodax(rp,mrot,27);
      }

  if (strcmp([demoMenu title],"lattice")==0) {
      geoqm(qpole,rl,64);
  sphcl(qpole,1.0e-4);
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadMatrix cellAt:i:j] setDoubleValue:qpole[i][j]];
      sphji(qdiag,qpole);
  sphcl(qpole,1.0e-3);
  sphcl(qdiag,1.0e-3);
	[quadPanel orderFront];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadEigenv cellAt:i:j] setDoubleValue:qdiag[i][j]];
	for (i=0; i<3; i++) for (j=0; j<3; j++) 
	   [[quadDiagon cellAt:i:j] setDoubleValue:qpole[i][j]];
      [self rotateMolecule:self];
      georf(rl,mrot,64);
      georf(rl,qdiag,64);
      if ([showAtoms state]) fsisf(rl,64,v);      
      if ([showAxes state]) geodax(rl,mrot,64);
      }

  if (strcmp([demoMenu title],"contour")==0) {
      cudem(cubic,11,11,11,0);
    for (i=0; i<10; i++) for (j=0; j<10; j++) for (k=0; k<10; k++)
      conth(cubic,0.40,i,j,k,v);
      }


  if (drawView==5) PSstroke();
 // if (drawView==5) [self flushWindow];

  }

  return self;
}

- clear:sender
/*
 * Clears the GeomView by emptying its Storage object of all points
 * and then redisplaying the GeomView.  This action is taken only if
 * the GeomView's state requires it.
 */
{
	if (needsClearing) {
		[points empty];
		[self display];
		needsClearing = NO;
	}
	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the GeomView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
	[super sizeTo:width :height];
	[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- registerPoint:(NXPoint *)aPoint
/*
 * Adds a point to the list the GeomView keeps in its Storage object. 
 */
{
	[points addElement:aPoint];
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the GeomView.  The GeomView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the GeomView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the GeomView, the GeomView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	GeomRect, cellRect;
   	char	buffer[100];

	[crossCursor set];
   	[self getBounds:&GeomRect];
 	NXSetRect(&cellRect, GeomRect.origin.x, GeomRect.origin.y, 100.0, 20.0);

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
	[self lockFocus];
        PSsetinstance(YES);
	do {
		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		sprintf(buffer, "(%d, %d)", (int)aPoint.x, (int)aPoint.y); 
  		[readOut setStringValue:buffer];
  		[readOut drawInside:&cellRect inView:self];
				PSsetgray(NX_BLACK);
//				drawCircle(aPoint.x, aPoint.y, 12.0);
//				PSmoveto(aPoint.x, aPoint.y);
//				PSlineto(aPoint.x+12.0, aPoint.y);
//				PSlineto(aPoint.x+12.0, aPoint.y+12.0);
//				PSlineto(aPoint.x, aPoint.y+12.0);
//				PSlineto(aPoint.x, aPoint.y);
				PSarc(aPoint.x, aPoint.y,12.0,0.0,360.0);
				PSstroke();
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
    /* on mouse-up, register point, inform delegate, and clean up state */
			[readOut setStringValue:""];
  			[readOut drawInside:&cellRect inView:self];
			[window flushWindow];
			if (NXPointInRect(&aPoint, &GeomRect)) {
				PSsetgray(NX_BLACK);
				drawCircle(aPoint.x, aPoint.y, radius);
				[window flushWindow];
				[self registerPoint:&aPoint];
				needsClearing = YES;
				if (delegate && [delegate respondsTo:@selector(PlotView:pointDidChange:)])
					[delegate PlotView:self pointDidChange:&aPoint];
			}
			looping = NO;
		}	
   	} while (looping && (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- setRadius:(float)aFloat
/*
 * Sets the value for the radius of the GeomView's points.
 */
{
	radius = aFloat;
	return self;
}


- (float)radius
/*
 * Returns the value for the radius of the GeomView's points.
 */
{
	return radius;
}


- read:(NXTypedStream *)stream
/*
 * Unarchives the GeomView.  Initializes four of the GeomView's instance 
 * variables to the values stored in the stream. 
 */
{
	[super read:stream];
	delegate = NXReadObject(stream);
	NXReadTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- write:(NXTypedStream *)stream
/*
 * Archives the GeomView by writing its important instance variables to the 
 * stream. 
 */
{
	[super write:stream];
	NXWriteObjectReference(stream, delegate);
	NXWriteTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- awake
/*
 * Finishes initializing a GeomView that has just been unarchived (see the 
 * read: method).
 */
{
	loadPSProcedures();
	return [super awake];
}


- (const char *)inspectorName
{
    return "GeomViewInspector";
}

- Geom:sender
/*
 * Responds to a Geom: message by asking the GeomView's delegate for a stream
 * containing the points to Geom.  It then extracts number pairs from the 
 * stream and creates NXPoint structures with them.  For each structure it 
 * creates, the GeomView sends itself a registerPoint: message to add the 
 * point to its list of points.  This simple parser ignores input it doesn't 
 * understand.
 */
{
	int	 	 c, sign, retval;
	float	 	 aNum;
	NXPoint	 	 aPoint;
	NXStream	*stream;
	BOOL		 processedLine = NO, gotFirst = NO, gotSecond = NO;
	
	if (delegate && [delegate respondsTo:@selector(PlotView:providePoints:)])
		[delegate PlotView:self providePoints:&stream];
	NXSeek(stream, 0, NX_FROMSTART);
	while ((c = NXGetc(stream)) != EOF){
		sign = 1;
		retval = getSeparator(stream);
		if (retval == 1) {
			c = NXGetc(stream);
		} else if (retval == -1) {
			break;
		}
		if (!NXIsDigit(c)) {
			if ((c == '-') && NXIsDigit(c = NXGetc(stream))){
					sign = -1;
			} else {
				while( (c != '\n') && (c != EOF)) {
					c = NXGetc(stream);
					processedLine = YES;
				}
			}
		}
		if (c == EOF)
			break;
		else if (processedLine) {
			processedLine = NO;
			continue;
		}
		aNum = getNumber(stream);
		if (!gotFirst) {
			aPoint.x = sign * aNum;
			gotFirst = YES;
		} else if (!gotSecond) {
			aPoint.y = sign * aNum;
			[self registerPoint:&aPoint];
			gotFirst = gotSecond = NO;
		} 
	}
	[self display];
	needsClearing = YES;
	return self;
}

int getSeparator(NXStream *stream)
/* 
 * Removes white space, commas, and plus signs from between 
 * number pairs.
 */
{
	int c, firstChar;
	
	NXUngetc(stream);
	c = firstChar = NXGetc(stream);
	while (NXIsSpace(c) || (c == ',') || (c == '+'))
		c = NXGetc(stream);
	/* 1 = ate something; 0 = ate nothing; -1 means EOF */
	if (c == firstChar)
		return 0;
	else if (c == EOF)
		return -1;
	NXUngetc(stream);
	return 1;
}

float getNumber(NXStream *stream)
/* 
 * Composes a floating point number from a string of number  
 * characters.
 */
{
	int c, i = 0;
	char temp[MAXNUMLENGTH];
	
	NXUngetc(stream);
	c = NXGetc(stream);
	while ((NXIsDigit(c) || (c == '.')) && (c != '\n') && (c != EOF)) {
		temp[i++] = c;
		c = NXGetc(stream);
	}
	NXUngetc(stream);
	temp[i] = 0;
	return (atof(temp));
}

@end

/* end GeomView.m */
