/* - - - - - M O L E C U L E   P R O G R A M S - - - - - */

/* molfc - generate a ferrocene molecule */
/* r1	iron radius			 */
/* r2   carbon radius			 */
/* r3	hydrogen radius			 */
/* r4   c-h bond			 */

void molfc(s,r1,r2,r3,r4) double s[][8], r1, r2, r3, r4; {
molpr(&s[0][0],1.0+r4,r3,1.3,30.0,30.0,30.0);	/* upper hydrogen ring	*/
molpr(&s[5][0],1.0,r2,1.0,30.0,30.0,30.0);	/* upper carbon ring	*/
s[10][0]=s[10][1]=s[10][2]=0.0; s[10][3]=r1;	/* iron atom		*/
s[10][4]=80.0; s[10][5]=-30.0; s[10][6]=100.0;
s[10][7]=r1*r1;
molps(&s[11][0],1.0,r2,-1.0,30.0,30.0,30.0);	/* lower carbon ring	*/
molps(&s[16][0],1.0+r4,r3,-1.3,30.0,30.0,30.0);	/* lower hydrogen ring  */
}

/* molpr - generate a pentagonal ring */
/* s[][8]  atom list	  */
/* r       radius of ring */
/* rr      radius of atom */
/* h       height         */
/* a,b,c   euler angles	  */

void molpr(s,r,rr,h,a,b,c) double s[][8], r, rr, h, a, b, c; {
int i;
double ph;
for (i=0, ph=0.0; i<5; i++,ph+=(72.0*3.14159)/180.0) {
  s[i][0]=r*cos(ph); s[i][1]=h; s[i][2]=r*sin(ph);
  s[i][3]=rr;
  s[i][4]=a; s[i][5]=b; s[i][6]=c;
  s[i][7]=rr*rr;
  }
}

/* molps - generate a staggered pentagonal ring */

void molps(s,r,rr,h,a,b,c) double s[][8], r, rr, h, a, b, c; {int i; double ph;
for (i=0, ph=(36.0*3.14159)/180.0; i<5; i++,ph+=(72.0*3.14159)/180.0) {
  s[i][0]=r*cos(ph); s[i][1]=h; s[i][2]=r*sin(ph);
  s[i][3]=rr;
  s[i][4]=a; s[i][5]=b; s[i][6]=c;
  s[i][7]=rr*rr;
  }
}

/* molhr - generate a hexagonal ring */

void molhr(s,r,rr,a,b,c) double s[][8], r, rr, a, b, c; {int i; double ph;
for (i=0, ph=0.0; i<6; i++,ph+=(60.0*3.14159)/180.0) {
  s[i][0]=r*cos(ph); s[i][1]=0.0; s[i][2]=r*sin(ph);
  s[i][3]=rr;
  s[i][4]=a; s[i][5]=b; s[i][6]=c;
  s[i][7]=rr*rr;
  }
}

/* molbe - generate a benzene molecule  */
/* s[12][8] parameter array		*/
/* ac       radius of carbon ring	*/
/* rc       radius of carbon atom	*/
/* ah       radius of hydrogen ring	*/
/* rh       radius of hydrogen atom	*/

void molbe(s,ac,rc,ah,rh) double s[][8], ac, rc, ah, rh; {
molhr(s[0],rc,ac, 30.0,50.0,40.0);
molhr(s[6],rh,ah,-60.0,40.0,30.0);
}

/* molsu - generate coordinates of a sulfur ring */

void molsu(s,radsu,bondsu,n) double s[][8], radsu, bondsu; int n; {int i;
for (i=0; i<n; i++) {
molsc(&s[i][0],&s[i][1],&s[i][2],bondsu,i,n);
s[i][3]=radsu;
s[i][4]=30.0;
s[i][5]=20.0;
s[i][6]=15.0;
s[i][7]=radsu*radsu;
}}

/* position an atom in a sulfur ring	*/
/*  p	position of the atom		*/
/*  n   number of atoms in the ring	*/
/*  d   radius of the ring		*/
/* al	angle between successive atoms	*/
/* be	bond angle between sulfur atoms	*/

void molsc(x,y,z,d,p,n) double *x, *y, *z, d; int p, n; {
double al, be, t;
// d=2.048;
al=6.2831852/((double) n);
be=(107.9*3.1415926)/360.0;
t=sin(be)/cos(al/2.0);
*x=d*sin(be)*cos(p*al)/sin(al);
*y=d*sin(be)*sin(p*al)/sin(al);
*z=(p%2==0?1.0:-1.0)*(d/2.0)*sqrt(1.0-t);
}

/* Generate a random assortment of n atoms */

void molra(s,n) double s[][8]; int n; {int i;
for (i=0; i<n; i++){
  s[i][0]=((double)(rand()%4096))/1024.0-2.0;
  s[i][1]=((double)(rand()%4096))/1024.0-2.0;
  s[i][2]=((double)(rand()%4096))/1024.0-2.0;
  s[i][3]=((double)(rand()%4096))/4096.0+0.1;
  s[i][4]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][5]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][6]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][7]=s[i][3]*s[i][3];
  }
}

/* Generate random parameters for the ith atom */

void molran(s,i) double s[][8]; int i; {
  s[i][0]=((double)(rand()%4096))/1024.0-2.0;
  s[i][1]=((double)(rand()%4096))/1024.0-2.0;
  s[i][2]=((double)(rand()%4096))/1024.0-2.0;
  s[i][3]=((double)(rand()%4096))/4096.0+0.1;
  s[i][4]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][5]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][6]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][7]=s[i][3]*s[i][3];
}

/* Fill a three dimensional lattice randomly */
/* p is probability in mils                  */
/* n is lattice width (not number of points) */

void molla(s,p,n) double s[][8]; int p, n; {
int    i, j, k, ii;
double dx;
  dx=3.0/((double)(n-1));
for (i=0; i<n; i++) for (j=0; j<n; j++) for (k=0; k<n; k++) {
  ii=i+n*(j+n*k);
  s[ii][0]=-1.5+((double)i)*dx;
  s[ii][1]=-1.5+((double)j)*dx;
  s[ii][2]=-1.5+((double)k)*dx;
  s[ii][3]=(rand()%1024)<p?0.4*dx:0.166;
  s[ii][4]=(((double)(rand()%4096))/4096.0)*180.0;
  s[ii][5]=(((double)(rand()%4096))/4096.0)*180.0;
  s[ii][6]=(((double)(rand()%4096))/4096.0)*180.0;
  s[ii][7]=s[ii][3]*s[ii][3];
}
}

/* spheres at the vertices of a tetrahedron */

void thedr(s) double s[][8]; {
int i;
  s[0][0]= 0.0;
  s[0][1]=-1.0;
  s[0][2]= 1.0;

  s[1][0]= 0.0;
  s[1][1]= 1.0;
  s[1][2]= 1.0;

  s[2][0]= 1.0;
  s[2][1]= 0.0;
  s[2][2]=-1.0;

  s[3][0]=-1.0;
  s[3][1]= 0.0;
  s[3][2]=-1.0;

for (i=0; i<4; i++) {
  s[i][3]=0.25+(((double)(rand()%4096))/4096.0)*0.82;
  s[i][4]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][5]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][6]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][7]=s[i][3]*s[i][3];
  }
}

/* spheres at the vertices of a regular tetrahedron */

void thon(s) double s[][8]; {
int i;
  s[0][0]= 0.0;
  s[0][1]=-1.0;
  s[0][2]= 1.0;
  s[0][3]= 0.5;

  s[1][0]= 0.0;
  s[1][1]= 1.0;
  s[1][2]= 1.0;
  s[1][3]= 0.5;

  s[2][0]= 1.0;
  s[2][1]= 0.0;
  s[2][2]=-1.0;
  s[2][3]= 0.5;

  s[3][0]=-1.0;
  s[3][1]= 0.0;
  s[3][2]=-1.0;
  s[3][3]= 0.5;

for (i=0; i<4; i++) {
  s[i][4]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][5]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][6]=(((double)(rand()%4096))/4096.0)*180.0;
  s[i][7]=s[i][3]*s[i][3];
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

