/* ----------------------------------------------------------------- */
/* Jacobi diagonalization personalized for [NN][DD][NN][DD] matrices */
/* ----------------------------------------------------------------- */

#import <math.h> 

/* make a jacobi rotation of the rows of the   		 */ 
/* matrix z through angle t in plane i0,i1, j0,j1  	 */
/* two columns will be affected                		 */

void jbyjr(z,i0,i1,j0,j1,th)
 double z[NN][DD][NN][DD];
 int i0, i1, j0, j1;
 double th;
 {
int    k0, k1;
double c, s, t;
s=sin(th); c=cos(th);
for (k0=0; k0<NN; k0++) for (k1=0; k1<DD; k1++) {
  t=z[k0][k1][i0][i1];
  z[k0][k1][i0][i1]= c*t+s*z[k0][k1][j0][j1];
  z[k0][k1][j0][j1]=-s*t+c*z[k0][k1][j0][j1];
  }
}

/* make a jacobi rotation of the columns of the 	*/ 
/* matrix z through angle t in plane i0,i1, j0,j1	*/
/* two rows will be affected                    	*/

void jbyjc(z,i0,i1,j0,j1,th)
 double z[NN][DD][NN][DD];
 int i0, i1, j0, j1;
 double th;
 {
int    k0, k1;
double c, s, t;
s=sin(th); c=cos(th);
for (k0=0; k0<NN; k0++) for (k1=0; k1<DD; k1++) {
  t=z[i0][i1][k0][k1];
  z[i0][i1][k0][k1]= c*t+s*z[j0][j1][k0][k1];
  z[j0][j1][k0][k1]=-s*t+c*z[j0][j1][k0][k1];
  }
}

/* diagonalize a symmetric matrix and record its	*/
/* eigenvectors (using Jacobi's method).         	*/
/* naturally, the matrix will be modified		*/
/* u[NN][DD][NN][DD] = matrix of eigenvectors		*/
/* v[NN][DD][NN][DD] = matrix being diagonalized	*/

void jbyji(u,v) double u[NN][DD][NN][DD], v[NN][DD][NN][DD]; {
int    i0, i1, j0, j1, k, l, ll;
double e, th;
for (i0=0; i0<NN; i0++) for (i1=0; i1<DD; i1++)
for (j0=0; j0<NN; j0++) for (j1=0; j1<DD; j1++)
   u[i0][i1][j0][j1]=0.0;
for (i0=0; i0<NN; i0++) for (i1=0; i1<DD; i1++)
   u[i0][i1][i0][i1]=1.0;
  ll=0;
for (k=0, e=1.0; k<10; k++, e/=3.0) {
  do {l=0;
    for (i0=0; i0<NN-1; i0++) for (i1=0; i1<DD; i1++)
    for (j0=i0+1; j0<NN; j0++) for (j1=0; j1<DD; j1++) {
      ll++; if (ll>500) return;
      if(v[i0][i1][j0][j1]>-e&&v[i0][i1][j0][j1]<e) continue; 
      th=0.5*atan2(v[i0][i1][j0][j1]+v[j0][j1][i0][i1],
	v[i0][i1][i0][i1]-v[j0][j1][j0][j1]);
      jbyjr(u,i0,i1,j0,j1,th);
      jbyjr(v,i0,i1,j0,j1,th);
      jbyjc(v,i0,i1,j0,j1,th);
      l=1;
      }
    }  while (l==1);
  }
 }

