
/* diagonalize an 11x11 symmetrical matrix by Jacobi's method */

#import <math.h> 

void elvjr(double z[11][11], int i, int j, double th);
void elvjc(double z[11][11], int i, int j, double th);
void elvji(double u[11][11], double v[11][11]);

/* make a jacobi rotation of the rows of the    */ 
/* matrix z through angle th in plane i, j      */
/* two columns will be affected                 */

void elvjr(z,i,j,th) double z[11][11]; int i, j; double th; {
int    k;
double c, s, t;
s=sin(th); c=cos(th);
for (k=0; k<11; k++) {
  t=z[k][i];
  z[k][i]= c*t+s*z[k][j];
  z[k][j]=-s*t+c*z[k][j];
  }
}

/* make a jacobi rotation of the columns of the */ 
/* matrix z through angle th in plane i, j      */
/* two rows will be affected                    */

void elvjc(z,i,j,th) double z[11][11]; int i, j; double th; {
int    k;
double c, s, t;
s=sin(th); c=cos(th);
for (k=0; k<11; k++) {
  t=z[i][k];
  z[i][k]= c*t+s*z[j][k];
  z[j][k]=-s*t+c*z[j][k];
  }
}

/* diagonalize a symmetric matrix and record its */
/* eigenvectors (using Jacobi's method).         */
/* u[11][11] = matrix of eigenvectors              */
/* v[11][11] = matrix being diagonalized           */

void elvji(u,v) double u[11][11], v[11][11]; {
int    i, j, k, l, ll;
double e, th;
for (i=0; i<11; i++) for (j=0; j<11; j++) u[i][j]=0.0;
for (i=0; i<11; i++) u[i][i]=1.0;
for (k=0, e=1.0; k<12; k++, e/=3.0) {
  ll=0;
  do {l=0;
    for (i=0; i<10; i++) for (j=i+1; j<11; j++) {
      ll++; if (ll>1000) break;
      if(v[i][j]>-e&&v[i][j]<e) continue; 
      th=0.5*atan2(v[i][j]+v[j][i],v[i][i]-v[j][j]);
      elvjr(u,i,j,th); elvjr(v,i,j,th); elvjc(v,i,j,th);
      l=1;
      }
    }  while (l==1);
//  printf("%3d, %6d\n",k,ll);
  }
 }

