
/* PEN.C				*/
/* Harold V. McIntosh, 01 November 1972 */
/* 1 November 1993 - "C" version	*/

# import <math.h>
# import "pen.h"


double pedb[PDM][PDB];			          /* diagonal band	*/
double pedy[PDM][PDM];			 	  /* dynamical matrix	*/
double penm[PDM][PDM];				  /* normal modes	*/
double pepl[8]={1.0,0.1,0.1,0.1,0.1,0.1,0.1,0.1}; /* parameter list	*/


# define penij(z,x,i,j) pencm(z[i][j],x)

/* add a multiple of one matrix to another */

void penau (z,x,f,y) double z[4][4], x[4][4], f, y[4][4]; {
int i, j;
for (i=0; i<4; i++) for (j=0; j<4; j++) z[i][j]=x[i][j]+f*y[i][j];
}

/* generate cyclical dynamical matrix from lth band of db */
/*   dm[n][n]   dynamical matrix to be constructed	  */
/*    db[][PDB]   matrix of individual rows		  */
/*         ll   row to be copied			  */

void pencd(dm,db,ll,n) double dm[][11][4][4], db[][PDB][4][4]; int ll, n; {
int    i, j, k, l;
if (n<PDB) return;
for (i=0; i<n; i++) for (j=0; j<n; j++) 
  for (k=0; k<4; k++) for (l=0; l<4; l++) dm[i][j][k][l]=0.0;
for (i=0; i<n; i++) for (j=i-3; j<i+4; j++) {
  for (k=0; k<4; k++) for (l=0; l<4; l++) 
	dm[i][(j+n)%n][k][l]=db[ll][j-i+3][k][l];
  }
}

/* copy 3x3 matrix */

void pencm(z,x) double z[4][4], x[4][4]; {int i, j; 
for (i=0; i<4; i++) for (j=0; j<4; j++) z[i][j]=x[i][j];
}

/* copy vector */

void pencv(z,x) double z[4], x[4]; {int i; 
for (i=0; i<4; i++) z[i]=x[i];
}

/* determinant of 3x3 matrix */

double  pende(z) double z[4][4]; {
return  z[0][0]*(z[1][1]*z[2][2]-z[2][1]*z[1][2])+
	z[0][1]*(z[1][2]*z[2][1]-z[1][0]*z[2][2])+
	z[0][2]*(z[2][1]*z[1][0]-z[2][0]*z[1][1]);
}

/* generate the diagonal band for uniform interactions */

void pendb(db,f,s) double db[PDM][PDB], f, s; {int i, j;
for (i=0; i<PDM; i++) {
  db[i][0]=s;
  db[i][1]=f;
  db[i][2]=-2.0*(f+s);
  db[i][3]=f;
  db[i][4]=s;
  }
}

/* dyadic product of two vectors */

void pendy(z,x,y) double z[4][4], x[4], y[4]; {int i, j;
for (i=0; i<4; i++) for (j=0; j<4; j++) z[i][j]=x[i]*y[j];
}

/* first neighbor stretching coefficients */ 

void penfs(db,sk,c,i) double db[][PDB][4][4], sk[][4], c; int i;{
double z[4][4], x[4];
  penst(x,sk,i,i+1);
  pendy(z,x,x);
  penau(db[i][4],db[i][4], c,z);
  penau(db[i][4],db[i][4],-c,z);
  penst(x,sk,i,i-1);
  pendy(z,x,x);
  penau(db[i][2],db[i][2], c,z);
  penau(db[i][4],db[i][4],-c,z);
}

/* invert a 3x3 matrix */

void penmi(z,x) double z[4][4], x[4][4]; {double d;
d=pende(x); if (d==0.0) return;
z[0][0]=(x[1][1]*x[2][2]-x[1][2]*x[2][1])/d;
z[0][1]=(x[0][1]*x[2][2]-x[2][1]*x[0][2])/d;
z[0][2]=(x[0][1]*x[1][2]-x[1][1]*x[0][2])/d;
z[1][0]=(x[1][2]*x[2][0]-x[1][0]*x[2][2])/d;
z[1][1]=(x[2][2]*x[0][0]-x[2][0]*x[0][2])/d;
z[1][2]=(x[1][0]*x[0][2]-x[0][0]*x[1][2])/d;
z[2][0]=(x[1][0]*x[2][1]-x[2][0]*x[1][1])/d;
z[2][1]=(x[2][0]*x[0][1]-x[0][0]*x[2][1])/d;
z[2][2]=(x[0][0]*x[1][1]-x[0][1]*x[1][0])/d;
}

/* matrix product */

void penmp(z,x,y) double z[4][4], x[4][4], y[4][4]; {
int    i, j, k;
double u[4][4], v[4][4];
pencm(u,x);
pencm(v,y);
for (i=0; i<4; i++) for (j=0; j<4; j++) {
  z[i][j]=0.0;
  for (k=0; k<4; k++) z[i][j]+=u[i][k]*v[k][j];
  }
}

/* matrix transpose */

void penmt(z,x) double z[4][4], x[4][4]; {
double t;
z[0][0]=x[0][0];
z[1][1]=x[1][1];
z[2][2]=x[2][2];
t=x[0][1];
z[0][1]=x[1][0];
z[1][0]=t;
t=x[0][2];
z[0][2]=x[2][0];
z[2][0]=t;
t=x[1][2];
z[1][2]=x[2][1];
z[2][1]=t;
}

/* negate a 3x3 matrix */

void penne(z,x) double z[4][4], x[4][4]; {int i, j;
for (i=0; i<4; i++) for (j=0; j<4; j++) z[i][j]=-x[i][j];
}

/* normalize vector */ 

void penno(z) double z[4]; {int i; double p;
for (i=0, p=0.0; i<4; i++) p+=z[i]*z[i];
p=sqrt(p);
if (p==0.0) return;
for (i=0; i<4; i++) z[i]/=p;
}

/* pensk sets up the molecular skeleton     */
/* sk[n][4]  atomic coordinates		    */
/*        r  radius of molecule		    */
/*        h  height of molecule		    */
/*	  a  angle between successive atoms */
/*        n  number of atoms in skeleton    */

void pensk(sk,r,h,a,n) double sk[][4], r, h, a; int n; {
int     i;
double ph, fi;
sk[0][0]=0.0;
sk[0][1]=h;
sk[0][2]=h;
if (n<=1) return;
ph=(a*3.14159)/180.0;
for (i=1; i<n; i++) {
  fi=ph*((double)(i));
  sk[i][0]= r*sin(fi);
  sk[i][1]=-sk[i-1][1];
  sk[i][2]= r*cos(fi);
  }
}

/* second neighbor stretching coeffieients */ 

void penss(db,sk,c,i) double db[][PDB][4][4], sk[][4], c; int i;{
double z[4][4], x[4];
  penst(x,sk,i,i+2);
  pendy(z,x,x);
  penau(db[i][5],db[i][5], c,z);
  penau(db[i][4],db[i][4],-c,z);
  penst(x,sk,i,i+2);
  pendy(z,x,x);
  penau(db[i][1],db[i][1], c,z);
  penau(db[i][4],db[i][4],-c,z);
}

/* stretching between particles i and j */

void penst(x,s,i,j) double x[4], s[][4]; int i, j; {int l;
for (l=0; l<4; l++) x[l]=s[l][i]-s[l][j];
penno(x);
}

/* generate the 4x4 transfer matrix z     */
/* from the lth diagonal strip taken from */
/* the array db, with eigenvalue e	  */

void pentm(z,db,e,l) double z[PTM][PTM], db[][PDB][4][4], e; int l; {
int    i, j;
double q[4][4], w[4][4];

penztm(z);
for (i=1; i<4; i++) z[i][i-1]=1.0;
penmi(q,db[6][l]);
penne(q,q);
for (i=1; i<6; i++) {
  penmp(w,q,db[i][l]);
  if (i==3) penau(w,w,-e,q);
//  penij(z,w,0,6-i);
  }
}

/* generate an nxn uniform dynamical matrix */

void penud(dm,db,l,n) double dm[21][21][4][4], db[][PDB][4][4]; int l, n; {
int    i, j, k;
double z[4][4];
penzdm(dm);
for (i=0; i<n; i++) for (j=0; j<n; j++) penij(dm,z,i,j);
for (i=0; i<n; i++) for (j=0; j<n; j++) {
  k=i+j-4;
  if (k>=0 && k<n) penij(dm,db[j][l],i,k);
  }
}

/* unit transfer matrix */

void penum(z) double z[PTM][PTM]; {int i, j;
for (i=0; i<PTM; i++) {
  for (j=0; j<PTM;j++) z[i][j]=0.0;
  z[i][i]=1.0;
  }
}

/* vector product */ 

void penvp(z,x,y) double z[4], x[4], y[4]; {double u[4], v[4]; 
  pencv(u,x);
  pencv(v,y);
  z[0]=u[1]*v[2]-u[2]*v[1];
  z[1]=u[2]*v[0]-u[0]*v[2];
  z[2]=u[0]*v[1]-u[1]*v[0];
}

/* zero transfer matrix */

void penztm(z) double z[PTM][PTM]; {int i, j;
for (i=0; i<PTM; i++) for (j=0; j<PTM; j++) z[i][j]=0.0;
}

/* zero dynamical matrix */

void penzdm(z) double z[PDM][PDM]; {int i, j;
for (i=0; i<PDM; i++) for (j=0; j<PDM; j++) z[i][j]=0.0;
}

