
/* SULF.C				*/
/* Harold V. McIntosh, 01 November 1972 */
/* 20 October 1993 - "C" version	*/

# import <math.h>
# include "sulf.h"

double suls[17][3];				  /* skeletal array	*/
double sudb[11][7][3][3];			  /* diagonal band	 */
double sudy[11][11][3][3];			  /* dynamical martrix	 */
double sunm[11][11][3][3];			  /* eigenvector martrix */
double supl[8]={1.0,0.1,0.1,0.1,0.1,0.1,0.1,0.1}; /* parameter list	 */

# define sulij(z,x,i,j) sulcm(z[i][j],x)

/* add a multiple of one matrix to another */

void sulau (z,x,f,y) double z[3][3], x[3][3], f, y[3][3]; {
int i, j;
for (i=0; i<3; i++) for (j=0; j<3; j++) z[i][j]=x[i][j]+f*y[i][j];
}

/* bending interaction between particles i, j, k */

void sulbe(x,y,s,i,j,k) double x[3], y[3], s[][3]; int i, j, k; {
int l;
double w[3];
for (l=0; l<3; l++) {
  x[l]=s[l][i]-s[l][j];
  y[l]=s[l][k]-s[l][j];
  }
sulvp(w,x,y);
sulvp(x,x,w);
sulvp(y,y,w);
sulno (x);
sulno (y);
}

/* generate cyclical dynamical matrix from lth band of db */
/*   dm[n][n]   dynamical matrix to be constructed	  */
/*    db[][7]   matrix of individual rows		  */
/*         ll   row to be copied			  */

void sulcd(dm,db,ll,n) double dm[][11][3][3], db[][7][3][3]; int ll, n; {
int    i, j, k, l;
if (n<7) return;
for (i=0; i<n; i++) for (j=0; j<n; j++) 
  for (k=0; k<3; k++) for (l=0; l<3; l++) dm[i][j][k][l]=0.0;
for (i=0; i<n; i++) for (j=i-3; j<i+4; j++) {
  for (k=0; k<3; k++) for (l=0; l<3; l++) 
	dm[i][(j+n)%n][k][l]=db[ll][j-i+3][k][l];
  }
}

/* copy 3x3 matrix */

void sulcm(z,x) double z[3][3], x[3][3]; {int i, j; 
for (i=0; i<3; i++) for (j=0; j<3; j++) z[i][j]=x[i][j];
}

/* copy vector */

void sulcv(z,x) double z[3], x[3]; {int i; 
for (i=0; i<3; i++) z[i]=x[i];
}

/* generate the kth diagonal band from the skeletal array */

void suldb(db,sk,pa,k) double db[][7][3][3], sk[][3], pa[]; int k; {
int    i, j;
double u[3][3], v[3][3];
for (j=0; j<7; j++) sulzm(db[k][j]);
sulfs(db,sk,pa[0],k);
sulss(db,sk,pa[1],k);
sulsb(db,sk,pa[3],k);
sults(db,sk,pa[2],k);
sultb(db,sk,pa[4],pa[4],k);
sultt(db,sk,pa[5],k);
//sulro(u,30.0,-k);
//for (i=0; i<7; i++) {
//  sulro(v,30.0,i);
//  sulmp(db[i][k],u,db[i][k]);
//  sulmp(db[i][k],db[i][k],v);
//  }
}

/* determinant of 3x3 matrix */

double  sulde(z) double z[3][3]; {
return  z[0][0]*(z[1][1]*z[2][2]-z[2][1]*z[1][2])+
	z[0][1]*(z[1][2]*z[2][1]-z[1][0]*z[2][2])+
	z[0][2]*(z[2][1]*z[1][0]-z[2][0]*z[1][1]);
}

/* dyadic product of two vectors */

void suldy(z,x,y) double z[3][3], x[3], y[3]; {int i, j;
for (i=0; i<3; i++) for (j=0; j<3; j++) z[i][j]=x[i]*y[j];
}

/* first neighbor stretching coefficients */ 

void sulfs(db,sk,c,i) double db[][7][3][3], sk[][3], c; int i;{
double z[3][3], x[3];
  sulst(x,sk,i,i+1);
  suldy(z,x,x);
  sulau(db[i][4],db[i][4], c,z);
  sulau(db[i][3],db[i][3],-c,z);
  sulst(x,sk,i,i-1);
  suldy(z,x,x);
  sulau(db[i][2],db[i][2], c,z);
  sulau(db[i][3],db[i][3],-c,z);
}

/* invert a 3x3 matrix */

void sulmi(z,x) double z[3][3], x[3][3]; {double d;
d=sulde(x); if (d==0.0) return;
z[0][0]=(x[1][1]*x[2][2]-x[1][2]*x[2][1])/d;
z[0][1]=(x[0][1]*x[2][2]-x[2][1]*x[0][2])/d;
z[0][2]=(x[0][1]*x[1][2]-x[1][1]*x[0][2])/d;
z[1][0]=(x[1][2]*x[2][0]-x[1][0]*x[2][2])/d;
z[1][1]=(x[2][2]*x[0][0]-x[2][0]*x[0][2])/d;
z[1][2]=(x[1][0]*x[0][2]-x[0][0]*x[1][2])/d;
z[2][0]=(x[1][0]*x[2][1]-x[2][0]*x[1][1])/d;
z[2][1]=(x[2][0]*x[0][1]-x[0][0]*x[2][1])/d;
z[2][2]=(x[0][0]*x[1][1]-x[0][1]*x[1][0])/d;
}

/* matrix product */

void sulmp(z,x,y) double z[3][3], x[3][3], y[3][3]; {
int    i, j, k;
double u[3][3], v[3][3];
sulcm(u,x);
sulcm(v,y);
for (i=0; i<3; i++) for (j=0; j<3; j++) {
  z[i][j]=0.0;
  for (k=0; k<3; k++) z[i][j]+=u[i][k]*v[k][j];
  }
}

/* matrix transpose */

void sulmt(z,x) double z[3][3], x[3][3]; {
double t;
z[0][0]=x[0][0];
z[1][1]=x[1][1];
z[2][2]=x[2][2];
t=x[0][1];
z[0][1]=x[1][0];
z[1][0]=t;
t=x[0][2];
z[0][2]=x[2][0];
z[2][0]=t;
t=x[1][2];
z[1][2]=x[2][1];
z[2][1]=t;
}

/* negate a 3x3 matrix */

void sulne(z,x) double z[3][3], x[3][3]; {int i, j;
for (i=0; i<3; i++) for (j=0; j<3; j++) z[i][j]=-x[i][j];
}

/* normalize vector */ 

void sulno(z) double z[3]; {int i; double p;
for (i=0, p=0.0; i<3; i++) p+=z[i]*z[i];
p=sqrt(p);
if (p==0.0) return;
for (i=0; i<3; i++) z[i]/=p;
}

/* kth power of the transformation matrix for successive atoms */

void sulro(z,a,k) double z[3][3], a; int k; {double p;
p=(((double)(k))*a*3.14159)/180.0;
sulzm(z);
z[0][0]= cos(p);
z[0][2]= sin(p);
z[1][1]=k%2==0?1.0:-1.0;
z[2][0]=-sin(p);
z[2][2]= cos(p);
}

/* second neighbor bending coefficient */

void sulsb(db,sk,c,i) double db[][7][3][3], sk[][3], c; int i; {
double z[3][3], x[3], y[3];
  sulbe(x,y,sk,i,i+1,i+2);
  suldy(z,x,y);
  sulau(db[i][5],db[i][5], c,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c,z);
  sulbe(x,y,sk,i,i-1,i-2);
  suldy(z,x,y);
  sulau(db[i][1],db[i][1], c,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c,z);
} 

/* sulsk sets up the molecular skeleton     */
/* sk[n][3]  atomic coordinates		    */
/*        r  radius of molecule		    */
/*        h  height of molecule		    */
/*	  a  angle between successive atoms */
/*        n  number of atoms in skeleton    */

void sulsk(sk,r,h,a,n) double sk[][3], r, h, a; int n; {
int     i;
double ph, fi;
sk[0][0]=0.0;
sk[0][1]=h;
sk[0][2]=h;
if (n<=1) return;
ph=(a*3.14159)/180.0;
for (i=1; i<n; i++) {
  fi=ph*((double)(i));
  sk[i][0]= r*sin(fi);
  sk[i][1]=-sk[i-1][1];
  sk[i][2]= r*cos(fi);
  }
}

/* second neighbor stretching coeffieients */ 

void sulss(db,sk,c,i) double db[][7][3][3], sk[][3], c; int i;{
double z[3][3], x[3];
  sulst(x,sk,i,i+2);
  suldy(z,x,x);
  sulau(db[i][5],db[i][5], c,z);
  sulau(db[i][3],db[i][3],-c,z);
  sulst(x,sk,i,i-2);
  suldy(z,x,x);
  sulau(db[i][1],db[i][1], c,z);
  sulau(db[i][3],db[i][3],-c,z);
}

/* stretching between particles i and j */

void sulst(x,s,i,j) double x[3], s[][3]; int i, j; {int l;
for (l=0; l<3; l++) x[l]=s[l][i]-s[l][j];
sulno(x);
}

/* third neighbor bending coefficients */ 

void sultb(db,sk,c1,c2,i) double db[][7][3][3], sk[][3], c1, c2; int i;{
double z[3][3], x[3], y[3];
  sulbe(x,y,sk,i,i+1,i+3);
  suldy(z,x,y);
  sulau(db[i][6],db[i][6], c1,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c1,z);
  sulbe(x,y,sk,i,i+2,i+3);
  suldy(z,x,y);
  sulau(db[i][6],db[i][6], c2,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c2,z);
  sulbe(x,y,sk,i,i-1,i-3);
  suldy(z,x,y);
  sulau(db[i][0],db[i][0], c1,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c1,z);
  sulbe(x,y,sk,i,i-2,i-3);
  suldy(z,x,y);
  sulau(db[i][0],db[i][0], c2,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c2,z);
}

/* generate the 18x18 transfer matrix z   */
/* from the lth diagonal strip taken from */
/* the array db, with eigenvalue e	  */

void sultm(z,db,e,l) double z[6][6][3][3], db[][7][3][3], e; int l; {
int    i, j;
double q[3][3], w[3][3];

sulzm(w);
for (i=1; i<6; i++) for (j=0; j<6; j++) sulij(z,w,i,j);
sulum(w);
for (i=1; i<6; i++) sulij(z,w,i,i-1);
sulmi(q,db[6][l]);
sulne(q,q);
for (i=1; i<6; i++) {
  sulmp(w,q,db[i][l]);
  if (i==3) sulau(w,w,-e,q);
  sulij(z,w,0,6-i);
  }
}

/* third neighbor stretching coefficients */ 

void sults(db,sk,c,i) double db[][7][3][3], sk[][3], c; int i;{
double z[3][3], x[3];
  sulst(x,sk,i,i+3);
  suldy(z,x,x);
  sulau(db[i][6],db[i][6], c,z);
  sulau(db[i][3],db[i][3],-c,z);
  sulst(x,sk,i,i-3);
  suldy(z,x,x);
  sulau(db[i][0],db[i][0], c,z);
  sulau(db[i][3],db[i][3],-c,z);
}

/* third neighbor twisting coefficient */

void sultt(db,sk,c,i) double db[][7][3][3], sk[][3], c; int i;{
double z[3][3], x[3], y[3];
  sultw(x,y,sk,i,i+1,i+2,i+3);
  suldy(z,x,y);
  sulau(db[i][6],db[i][6], c,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c,z);
  sultw(x,y,sk,i,i-1,i-2,i-3);
  suldy(z,x,y);
  sulau(db[i][0],db[i][0], c,z);
  suldy(z,x,x);
  sulau(db[i][3],db[i][3],-c,z);
} 

/* twisting interaction between particles i, j, k, l */

void sultw(x,y,s,i,j,k,l) double x[3], y[3], s[][3]; int i, j, k, l; {
int m; double w[3];
for (m=0; m<3; m++) {
  x[m]=s[m][i]-s[m][j];
  y[m]=s[m][l]-s[m][k];
  w[m]=s[m][j]-s[m][k];
  }
sulvp(x,x,w);
sulvp(y,y,w);
sulno(x);
sulno(y);
}

/* generate an nxn uniform dynamical matrix */

void sulud(dm,db,l,n) double dm[21][21][3][3], db[][7][3][3]; int l, n; {
int    i, j, k;
double z[3][3];
sulzm(z);
for (i=0; i<n; i++) for (j=0; j<n; j++) sulij(dm,z,i,j);
for (i=0; i<n; i++) for (j=0; j<n; j++) {
  k=i+j-4;
  if (k>=0 && k<n) sulij(dm,db[j][l],i,k);
  }
}

/* unit matrix */

void sulum(z) double z[3][3]; {int i, j;
for (i=0; i<3; i++) {
  for (j=0; j<3;j++) z[i][j]=0.0;
  z[i][i]=1.0;
  }
}

/* vector product */ 

void sulvp(z,x,y) double z[3], x[3], y[3]; {double u[3], v[3]; 
  sulcv(u,x);
  sulcv(v,y);
  z[0]=u[1]*v[2]-u[2]*v[1];
  z[1]=u[2]*v[0]-u[0]*v[2];
  z[2]=u[0]*v[1]-u[1]*v[0];
}

/* print the diagonal band of the dynamical matrix */

void sulwb(z,n) double z[][7][3][3]; int n; {
int i, j, k, l;
printf("diagonal band of the dynamical matrix\n");
for (i=0; i<n; i++) {
  for (j=0; j<3; j++) {
    for (k=0; k<7; k++) {
      for (l=0; l<3; l++) printf("%6.3f ",z[i][k][j][l]);
      printf(", ");
      }
    printf("\n");
    }
  printf("\n");
  }
}

/* write the 3x3 matrix z with label c on the teletype */

void sulwm(z,c) double z[3][3]; char *c; {int i, j;
printf("%c\n",c);
for (i=0; i<3; i++) {
  for (j=0; j<3; j++) printf("%6.3f ",z[i][j]);
  printf("\n");
  }
printf("\n");
}

/* zero matrix */

void sulzm(z) double z[3][3]; {int i, j;
for (i=0; i<3; i++) for (j=0; j<3; j++) z[i][j]=0.0;
}

